##########################################################################
#                                                                        #
#           copyright (c) 2003 ITB, Humboldt-University Berlin           #
#           written by: Raphael Ritz, r.ritz@biologie.hu-berlin.de       #
#                                                                        #
##########################################################################

"""Article reference main class"""

from types import StringTypes

from AccessControl import ClassSecurityInfo
from Products.CMFCore.permissions \
     import View, ModifyPortalContent
from Products.CMFCore.utils import getToolByName

from Products.CMFBibliographyAT.config import CMFBAT_USES_LINGUAPLONE
if CMFBAT_USES_LINGUAPLONE:
    from Products.LinguaPlone.public import Schema
    from Products.LinguaPlone.public import StringField, BooleanField
    from Products.LinguaPlone.public import StringWidget, BooleanWidget
else:
    from Products.Archetypes.public import Schema
    from Products.Archetypes.public import StringField, BooleanField
    from Products.Archetypes.public import StringWidget, BooleanWidget
from Products.Archetypes.public import registerType

from Products.CMFBibliographyAT.content.base import BaseEntry
from Products.CMFBibliographyAT.config import PROJECTNAME
from Products.CMFBibliographyAT.content.schemata \
    import HeaderSchema, AuthorSchema, CoreSchema, TrailingSchema
from Products.CMFBibliographyAT.content.fields \
    import publisherField, addressField, editionField, volumeField, \
           numberField, seriesField, editor_flagField


AuthorTrailingSchema = Schema((
    editor_flagField,
    ))

SourceSchema = Schema((
    publisherField,
    addressField,
    editionField,
    volumeField,
    numberField,
    seriesField,
    StringField('isbn',
        searchable=1,
	default='',
	is_duplicates_criterion=True,
        widget=StringWidget(
            label="ISBN Number",
            label_msgid="label_isbn",
            description="The book's ISBN number.",
            description_msgid="help_isbn",
            i18n_domain="cmfbibliographyat",
	),
    ),
))
BookSchema = HeaderSchema.copy() + AuthorSchema.copy() + AuthorTrailingSchema.copy() + \
             CoreSchema.copy() + SourceSchema.copy() + TrailingSchema.copy()
BookSchema.get('authors').required = 1
BookSchema.get('publication_year').required = 1
BookSchema.get('publisher').required = 1

# the default AT 'description' field shall be invisible, it is kept in sync with the 'abstract' field
BookSchema.get('description').widget.visible = {'view': 'invisible', 'edit': 'invisible', }


class BookReference(BaseEntry):
    """ content type to make reference to a book.
    """
    archetype_name = "Book Reference"
    source_fields = ('publisher', 'address', 'edition', 'volume', 'number', 'isbn',)

    security = ClassSecurityInfo()

    schema = BookSchema

    actions = BaseEntry.actions + ({
        'id': 'setDetails',
        'name': 'Set Details from Amazon',
        'action': 'string:${object_url}/setDetailsFromISBN',
        'permissions': (ModifyPortalContent,),
        'condition': 'python: (object.getIsbn() != "") and getattr(portal, "amazon_tool", None) and getattr(portal, "amazon_tool").hasValidLicenseKey()',
        },
    )

    security.declareProtected(View, 'pre_validate')
    def pre_validate(self, REQUEST, errors):
    
	amazon_tool = getToolByName(self, 'amazon_tool', None)
	if (self.ISBN() or (REQUEST.form.has_key('isbn') and REQUEST.form['isbn'])) and ((self.Authors() == 'No names specified') and not self.getMember_publication_authors() and not self.Title()) and amazon_tool and amazon_tool.hasValidLicenseKey():

	    self.Schema()['isbn'].set(self, value=REQUEST.form['isbn'])
	    self.setDetailsFromISBN(is_new_object=True)
	    
	    # for BaseEntry.pre_validate we need at least authors in REQUEST.form
	    REQUEST.form['authors'] = self.Schema()['authors']._to_dict(self.Schema()['authors'].get(self) )
	
    	    # to ship around required schema fields we need these as well (especially on BookReference creation)
	    REQUEST.form['publication_year'] = self.getPublication_year()
	    REQUEST.form['publication_month'] = self.getPublication_month()
	    REQUEST.form['title'] = self.Title()
	    REQUEST.form['publisher'] = self.getPublisher()
	
	BaseEntry.pre_validate(self, REQUEST, errors)
	    
    security.declareProtected(View, 'Source')
    def Source(self):
        """ the default book source format
        """
        try:

            return self.BookSource()

        except AttributeError:

	    bs_tool = getToolByName(self, 'portal_bibliostyles', None)
	    
            publisher = self.getPublisher()
            address   = self.getAddress()
            edition   = self.getEdition()
	    volume    = self.getVolume()
	    number    = self.getNumber()
	    series    = self.getSeries()
            isbn      = self.getIsbn()
            
            source = publisher
            if address: source += ', %s' % address
            if volume: 
		source += ', vol. %s' % volume
        	if number: source += '(%s)' % number
            
	    if edition: 
		if bs_tool:
		    source += ', %s' % bs_tool.formatEdition(edition, abbreviate=True)
		else:
		    source += ',  %s ed.' % edition
            if source and (source[-1] != '.'):
	        source += '.'
	    if series: source += ' %s.' % series
            if isbn: source += ' (ISBN: %s).' % isbn
            
	    if source and (source[-1] not in '.!?'):
	        source += '.'
		
            return source

    security.declareProtected(View, 'ISBN')
    def ISBN(self):
        """ the ISBN number
        """
        return self.getIsbn()

    security.declareProtected(ModifyPortalContent, 'setDetailsFromISBN')
    def setDetailsFromISBN(self, isbn=None, is_new_object=False, REQUEST=None):
        """ Get details from Amazon using the AmazonTool.
            If isbn == None, try to use the asin (isbn) already on the object.
        """
        atool = getToolByName(self, 'amazon_tool')
        if isbn is None:
            isbn = self.getIsbn()
        if not isbn:
            raise Exception('Bad ISBN')
        result = atool.searchByASIN(asin=isbn, locale='us')[0]
        self.setTitle(result.ProductName)
        # result.Authors can look like:
        #   u'Joseph S. Nye' or
        #   [u'Peter A. Hall', u'David W. Soskice']
        authors = []
        if isinstance(result.Authors, StringTypes):
            result.Authors = [result.Authors]
        for each in result.Authors:
            author = {}
            l = each.split(' ')
            author['firstname'] = l[0]
            if len(l) == 3:
                author['middlename'] = l[1]
                author['lastname'] = l[2]
            else:
                author['middlename'] = ''
                author['lastname'] = l[1]
            authors.append(author)
        self.setAuthors(authors)
        # result.ReleaseDate looks like:
        #   u'01 December, 2001' or
        #   u'May, 2004'
        l = result.ReleaseDate.split(', ')
	year = l[1]
        if len(l) >= 2:
            month = l[0].split(' ')[1]
        else:
            month = l[0]
        self.setPublication_year(year)
        self.setPublication_month(month)
        self.setPublisher(result.Manufacturer)
	if not is_new_object:
	    self.reindexObject()

        if REQUEST is not None:
            self.REQUEST.RESPONSE.redirect(self.absolute_url())


registerType(BookReference, PROJECTNAME)



##########################################################################
#                                                                        #
#           copyright (c) 2003 ITB, Humboldt-University Berlin           #
#           written by: Raphael Ritz, r.ritz@biologie.hu-berlin.de       #
#                                                                        #
##########################################################################

"""Mastersthesis reference main class"""

from AccessControl import ClassSecurityInfo
from Products.CMFCore.permissions import View

from Products.CMFBibliographyAT.config import CMFBAT_USES_LINGUAPLONE
if CMFBAT_USES_LINGUAPLONE:
    from Products.LinguaPlone.public import Schema
    from Products.LinguaPlone.public import StringField, StringWidget
else:
    from Products.Archetypes.public import Schema
    from Products.Archetypes.public import StringField, StringWidget
from Products.Archetypes.public import registerType

from Products.CMFBibliographyAT.config import PROJECTNAME
from Products.CMFBibliographyAT.content.base import BaseEntry
from Products.CMFBibliographyAT.content.schemata \
    import HeaderSchema, AuthorSchema, CoreSchema, TrailingSchema
from Products.CMFBibliographyAT.content.fields \
    import schoolField, addressField, typeField


SourceSchema = Schema((
    schoolField,
    typeField,
    addressField,
     ))

MastersthesisSchema = HeaderSchema.copy() + AuthorSchema.copy() + CoreSchema.copy() +  \
                      SourceSchema.copy() + TrailingSchema.copy()
MastersthesisSchema.get('authors').required = 1			      
MastersthesisSchema.get('publication_year').required = 1
MastersthesisSchema.get('school').required = 1		

# the default AT 'description' field shall be invisible, it is kept in sync with the 'abstract' field
MastersthesisSchema.get('description').widget.visible = {'view': 'invisible', 'edit': 'invisible', }

class MastersthesisReference(BaseEntry):
    """ content type to make reference to a masters thesis.
    """
    security = ClassSecurityInfo()
    archetype_name = "Mastersthesis Reference"
    source_fields = ('school', 'address', 'publication_type',)

    schema = MastersthesisSchema

    security.declareProtected(View, 'Source')
    def Source(self):
        """ the default masters thesis source format
        """
        try:

            return self.MastersthesisSource()

        except AttributeError:

            school   		= self.getSchool()
            publication_type   	= self.getPublication_type()
            address  		= self.getAddress()
            
            if publication_type:
		source = publication_type
	    else:
		source = 'Master thesis'	
	    if school: source += ', %s' % school
	    if address: source += ', %s' % address
	    
            return source + '.'

registerType(MastersthesisReference, PROJECTNAME)



from AccessControl import ClassSecurityInfo
from Products.CMFCore.permissions import ManagePortal, ManageProperties
from Products.CMFCore.utils import getToolByName

from Products.CMFBibliographyAT.config import CMFBAT_USES_LINGUAPLONE
if CMFBAT_USES_LINGUAPLONE:
    from Products.LinguaPlone.public import Schema, \
	 PrimaryFieldMarshaller, FileField, FileWidget
else:
    from Products.Archetypes.public import Schema, \
	 PrimaryFieldMarshaller, FileField, FileWidget
from Products.Archetypes.public import registerType

from Products.ATContentTypes.content.folder import ATBTreeFolder
from Products.ATContentTypes.content.folder import ATBTreeFolderSchema
from Products.ATContentTypes.content.file import ATFile
from Products.ATContentTypes.content.schemata import ATContentTypeSchema
from Products.ATContentTypes.content.schemata import finalizeATCTSchema

from Products.CMFBibliographyAT.config \
     import PROJECTNAME, USE_EXTERNAL_STORAGE
from folder import associateToBibliographyFolder

if USE_EXTERNAL_STORAGE:
    from Products.ExternalStorage.ExternalStorage import ExternalStorage as Storage
else:
    from Products.Archetypes.Storage import AttributeStorage as Storage

finalizeATCTSchema(ATBTreeFolderSchema, folderish=True, moveDiscussion=False)

PdfFolderSchema = ATBTreeFolderSchema.copy()
PdfFolderSchema['id'].widget.visible['edit'] = 'invisible'

PdfBaseSchema = ATContentTypeSchema.copy()
PdfBaseSchema['id'].widget.condition = 'python: not object.getBibFolder().getSynchronizePdfFileAttributes()'
PdfBaseSchema['title'].required = False
PdfBaseSchema['title'].widget.visible['edit'] = 'invisible'
PdfBaseSchema['description'].widget.visible['edit'] = 'invisible'
PdfBaseSchema['relatedItems'].widget.visible['edit'] = 'invisible'
PdfBaseSchema['allowDiscussion'].widget.visible['edit'] = 'invisible'

PdfFileSchema = PdfBaseSchema + Schema((
    FileField('file',
              #required=True,
              primary=True,
              languageIndependent=True,
              default_content_type = "application/pdf",
              storage = Storage(),
	      # validators = (('isNonEmptyFile', V_REQUIRED),),
              widget = FileWidget(
                        label= "Printable File",
                        label_msgid = "label_upload_pdffile_from_bibrefitem",
                        description = "If not in conflict with any copyright issues, use this field to upload a printable version (PDF file) of the referenced resource.",
                        description_msgid = "help_upload_pdffile_from_bibrefitem",
                        i18n_domain = "cmfbibliographyat",
                        show_content_type = False,)),
    ) , marshall=PrimaryFieldMarshaller()
    )

finalizeATCTSchema(PdfFileSchema)

class PdfFolder(ATBTreeFolder):
    """CMFBib's specialized folder for holding printable (pdf) files"""

    schema = PdfFolderSchema

    content_icon   = 'bibliography_pdffolder.png'
    portal_type    = 'PDF Folder'
    archetype_name = 'PDF Folder'

    allowed_content_types = ()
    filter_content_types = True
    
    _assoc_bibliography_folder = None

    actions = (
	{
	'id'		: 'local_roles',
	'name'          : 'Sharing',
	'action'        : 'string:${object_url}/folder_localrole_form',
	'permissions'   : (ManageProperties,),
	'condition'     : 'python: object.portal_membership.checkPermission("Manage Properties", object)',
	},
    )
    
    typeDescription= "A specialized folder for holding printable (pdf) files"

    security       = ClassSecurityInfo()

    # play nice with the bibfolders custom listing
    publication_year = "0001" # should suffice to be always last

    # this method is provided as a function in CMFBAT's folder module
    associateToBibliographyFolder = associateToBibliographyFolder
         												
    def Authors(self, *args, **kw):
        """ compatibility method """
        return "Printable files"

    def Title(self):
        """ the Title"""
        return "PDFs"

    def getPublication_year(self):
        """ compatibility method """
        return "as available"

    def Source(self):
        """ compatibility method """
        return ""

    def nextId(self, testid):
	bib_tool = getToolByName(self, 'portal_bibliography')
	return bib_tool.nextId(testid)
	
    security.declareProtected(ManagePortal, 'allowPdfFileCreation')
    def allowPdfFileCreation(self):
	""" allow PDF file creation
	"""
	return self.manage_pdfFileCreationPolicy(enable=True)
	
    security.declareProtected(ManagePortal, 'disallowPdfFileCreation')
    def disallowPdfFileCreation(self):
	""" disallow PDF file creation
	"""
	return self.manage_pdfFileCreationPolicy(enable=False)
	
    security.declareProtected(ManagePortal, 'manage_pdfFileCreationPolicy')
    def manage_pdfFileCreationPolicy(self, enable=False):
	""" allow/disallow PDF File creation in PDF Folders...
	"""
	disable = not enable
	types_tool = getToolByName(self, 'portal_types')
	
	if enable:
    	    pdf_folder_fti = types_tool[self.portal_type]
	    if 'PDF File' not in pdf_folder_fti.allowed_content_types:
		pdf_folder_fti.allowed_content_types += ('PDF File',)

	if disable:
	    pdf_folder_fti = types_tool[self.portal_type]
	    pdf_folder_allowed_types = list(pdf_folder_fti.allowed_content_types)
	    for ct_idx in range(len(pdf_folder_allowed_types)-1, -1, -1):
		if pdf_folder_allowed_types[ct_idx] == 'PDF File':
		    del pdf_folder_allowed_types[ct_idx]
	    pdf_folder_fti.allowed_content_types = tuple(pdf_folder_allowed_types)

    def getBibFolder(self):
    
	return self.aq_inner.aq_parent

    def manage_beforeDelete(self, item, container):
	""" do some cleaning up before we leave the world """

	# manage_beforeDelete will be deprecated in Zope 2.11+
    	bibfolder = self.getBibFolder()
	bibfolder._assoc_pdf_folder = None
	ATBTreeFolder.manage_beforeDelete(self, item, container)

registerType(PdfFolder, PROJECTNAME)

class PdfFile(ATFile):
    """
    A restricted file type to hold printable (pdf) files only
    Get's its title and description from the associated reference
    """

    schema = PdfFileSchema

    content_icon   = 'bibliography_pdffile.png'
    portal_type    = 'PDF File'
    archetype_name = 'PDF File'
    global_allow = False
    assocMimetypes = ('application/pdf', )

    security       = ClassSecurityInfo()
    
    actions = (
	{
	'id'		: 'local_roles',
	'name'          : 'Sharing',
	'action'        : 'string:${object_url}/folder_localrole_form',
	'permissions'   : (ManageProperties,),
	'condition'     : 'python: object.portal_membership.checkPermission("Manage Properties", object)',
	},
    )
    
    def Title(self):
        """Override with the title from the associated reference"""
        refs = self.getBRefs('printable_version_of')
        if not (refs and refs[0]):
            return self.getId()
        else:
            return refs[0].Title()

    def Description(self):
        """Override with authors and source from the associated reference"""
        refs = self.getBRefs('printable_version_of')
        if not refs:
            return "No associated reference"
        ref = refs[0]
        return "PDF of %s (%s): %s Located at portal path: %s" % (ref.Authors(),
                                       ref.getPublication_year(),
                                       ref.Source(),
				       ref.absolute_url(),
                                       )

    def getBibFolder(self):

	return self.aq_inner.aq_parent.aq_inner.aq_parent

    def getPdfFolder(self):
    
	return self.aq_inner.aq_parent
	
registerType(PdfFile, PROJECTNAME)



from Globals import package_home
from Products.Archetypes.public import *
from Products.CMFCore import utils
from Products.CMFCore.DirectoryView import registerDirectory
import os, os.path

from config import *

registerDirectory(SKINS_DIR, GLOBALS)

def initialize(context):
    ##Import Types here to register them
    import BibFolder
    import Book
    import Article
    import Chapter
    import Journal
    import TechnicalReport
    import Thesis
    import Proceedings
    import ConferencePaper
    import NewsArticle
    import Standard

    content_types, constructors, ftis = process_types(
        listTypes(PROJECTNAME),
        PROJECTNAME)
    
    utils.ContentInit(
        PROJECTNAME + ' Content',
        content_types      = content_types,
        permission         = ADD_CONTENT_PERMISSION,
        extra_constructors = constructors,
        fti                = ftis,
        ).initialize(context)




from Products.bibTypes.config import *
from Products.Archetypes.public import *
from Products.CMFCore import CMFCorePermissions
from Globals import InitializeClass

Base_Schema = Schema((

    StringField('Abstract',
              searchable=1,
              required=0,
              edit_accessor="editAbstract",
              widget=TextAreaWidget(description="Abstract of the item.",
                                     label="Abstract",
                                     rows=5),
              ),

))

Trailing_Schema = Schema((

    StringField('PublicationUrl',
                required=0,
                searchable=0,
                validators=('isURL',),
                widget=StringWidget(description="The URL to get to the full text of the item.",
                                    label="URL",),
                ),

    StringField('Alt_id',
                required=0,
                searchable=0,
                widget=StringWidget(description="Enter a releveant alternative ID reference, such as PubMed number.",
                                    label="Alternative ID",),
                ),    

    BooleanField('personal_entry',
                 required=0,
                 searchable=0,
                 default=0,
                 widget=BooleanWidget(description="Is the entry a personal resource?",
                                     label="Personal Entry",),
                 ),
    
    ))

base_actions = (
    {
    'id' : 'edit',
    'name' : 'Edit',
    'action' : 'portal_form/base_edit',
    'permissions' : (CMFCorePermissions.ModifyPortalContent, )
    },
    {
    'id' : 'properties',
    'name' : 'Properties',
    'action' : 'portal_form/base_metadata',
    'permissions' : (CMFCorePermissions.ModifyPortalContent, )
    },
    {
    'id' : 'export',
    'name' : 'Export',
    'action' : 'portal_form/export',
    'permissions' : (CMFCorePermissions.View, )
    },
    )

class BibSchema(BaseContent):

    def getAbstract(self):
        # the accessor for dublin core metadata is a bit inconsistent
        # with Archetype
        return self.Description()

    def setAbstract(self, val):
        return self.setDescription(val)

    def editAbstract(self, **kw):
        if hasattr(self,"description"):
            return self.description
        else:
            return ""
    

InitializeClass(BibSchema)    


from AccessControl import ClassSecurityInfo
from ComputedAttribute import ComputedAttribute
from ZPublisher.HTTPRequest import FileUpload
from Products.Archetypes.public import *

simplememberschema = BaseSchema.copy() + Schema((
    StringField('fullname',
                widget = StringWidget(description = "Enter a contact person.")
                ),
    ))

memberschema = BaseSchema.copy() + Schema((
    StringField('memberId',
                widget = StringWidget(description = "Member ID.")
                ),
    StringField('firstname',
                widget = StringWidget(description = "Firstname.")
                ),
    StringField('middlename',
                widget = StringWidget(description = "Middlename.")
                ),
    StringField('lastname',
                widget = StringWidget(description = "Lastname.")
                ),
    StringField('homepage',
                widget = StringWidget(description = "Homepage.")
                ),
    ))

TEXT = 'file data'

class File(FileUpload):
    '''Dummy upload object
       Used to fake uploaded files.
    '''
    __allow_access_to_unprotected_subobjects__ = 1
    filename = 'dummy.txt'
    data = TEXT
    headers = {}

    def __init__(self, filename=None, data=None, headers=None):
        if filename is not None:
	    self.filename = filename
	    if data is not None:
	        self.data = data
	    if headers is not None:
	        self.headers = headers

    def seek(self, *args): pass
    def tell(self, *args): return 1
    def read(self, *args): return self.data

def getMemberInfo(id):

    return {'fullname': 'Fullname', 'id': 'Member ID', 'homepage': 'home page','email': 'email address'} 

class SimpleTestMemberType(BaseContent):
    """A simple test type"""

    id = 'simple_test_member_type'
    schema = simplememberschema
    meta_type = 'Simple Test Member Type'

    security = ClassSecurityInfo()

registerType(SimpleTestMemberType)

class TestMemberType(BaseContent):
    """A test type"""

    id = 'test_member_type'
    schema = memberschema
    meta_type = 'Test Member Type'

    security = ClassSecurityInfo()
    
    def Authors(self):
	if self.getMiddlename():
	    return '%s, %s %s' % (self.getLastname(), self.getFirstname(), self.getMiddlename())
	else:
	    return '%s, %s' % (self.getLastname(), self.getFirstname())

    getAuthors = ComputedAttribute('Authors')

registerType(TestMemberType)



##########################################################################
#                                                                        #
#           copyright (c) 2005 ITB, Humboldt-University Berlin           #
#           written by: Raphael Ritz, r.ritz@biologie.hu-berlin.de       #
#                                                                        #
##########################################################################

from Products.PortalTransforms.interfaces import itransform

class BibawareToHTML:
    """Transforms the mime-type back to text/html"""
    __implements__ = itransform
    __name__ = "bibaware_to_html"
    inputs = ('text/x-html-bibaware',)
    output = "text/html"
    
    def __init__(self, name=None):
        if name:
            self.__name__ = name

    def name(self):
        return self.__name__

    def convert(self, data, idata, filename=None, **kwargs):
        """
        simple identity to transform the mimetype only
        """
        idata.setData(data)
        return idata

def register():
    return BibawareToHTML()

def initialize():
    engine = getToolByName(portal, 'portal_transforms', None)
    if engine is not None:
        engine.registerTransform(register())



# inspired from PlacelessTranslation - utils.py
#    Copyright (C) 2001, 2002, 2003 Lalo Martins <lalo@laranja.org>,

from types import UnicodeType
from zLOG import LOG, INFO, BLATHER, PROBLEM, WARNING

def log(msg, severity=INFO, detail='', error=None):
    if type(msg) is UnicodeType:
        msg = msg.encode(sys.getdefaultencoding(), 'replace')
    if type(detail) is UnicodeType:
        detail = detail.encode(sys.getdefaultencoding(), 'replace')
    LOG('CMFBibliographyAT', severity, msg, detail, error)

# added some more routines...

## a poor-mans approach to fixing unicode issues :-(
_default_encoding = 'utf-8'

def _encode(s, encoding=_default_encoding):
    try:
        return s.encode(encoding)
    except (TypeError, UnicodeDecodeError, ValueError):
        return s
    
def _decode(s, encoding=_default_encoding):
    try:
        return unicode(s, encoding)
    except (TypeError, UnicodeDecodeError, ValueError):
        return s




#######################################################
#                                                     #
#    Copyright (C), 2004, Raphael Ritz                #
#    <r.ritz@biologie.hu-berlin.de>                   #
#                                                     #
#    Humboldt University Berlin                       #
#                                                     #
# Copyright (C), 2005, Logilab S.A. (Paris, FRANCE)   #
# http://www.logilab.fr/ -- mailto:contact@logilab.fr #
#                                                     #
#######################################################

__revision__ = '$Id:  $'


import os, sys

if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))

from Testing import ZopeTestCase
from Products.CMFPlone.tests import PloneTestCase
from Products.CMFBibliographyAT.tests import setup
from ZODB.PersistentList import PersistentList
from Globals import PersistentMapping



#ZopeTestCase.installProduct('DeadlockDebugger')
#import Products.DeadlockDebugger

class TestBibliographyFolder(PloneTestCase.PloneTestCase):
    '''Test the BibliographyFolder Import and duplication handling'''
    
    def afterSetUp(self):
        self._refreshSkinData()
        self.bf = self.getEmptyBibFolder()
        bib_source = open(setup.BIBTEX_TEST_BIB, 'r').read()
        self.bf.processImport(bib_source, 'bibtex_test.bib')

    #FIXME duplicate from testBibliographyFolder - REFACTOR
    def getEmptyBibFolder(self, ident ='bib_folder'):
        uf = self.folder
        uf.invokeFactory(type_name = "BibliographyFolder",
                         id = ident )
        return getattr(uf, ident)
    
    def testBibtexImportDuplicate(self):
        bibtool = self.portal.portal_bibliography
        filter = {'portal_type': bibtool.getReferenceTypes(), }

	self.insertDuplicates()
        self.assertEquals(len(self.bf.contentIds(filter=filter)), 3 ,
                          'Duplicate Entry was introduced, expected 3 unique entries')

    def insertDuplicates(self):
        bib_source = open(setup.BIBTEX_TEST_BIB_DUP, 'r').read()
        self.bf.processImport(bib_source, 'bibtex_test_duplicates.bib')

    def insertSameImportFile(self):
        bib_source = open(setup.BIBTEX_TEST_BIB, 'r').read()
        self.bf.processImport(bib_source, 'bibtex_test.bib')


    def testBibtexImportDuplicate_Access(self):
        self.insertDuplicates()

        self.assertNotEqual(self.bf.getDuplicatesFolder().contentValues(), [])

        dup_entry = self.bf.getDuplicatesFolder().contentValues()[0]
        self.assertEquals(dup_entry.Title(), 'Programming Python.')
        self.assertEquals(len(dup_entry.getIs_duplicate_of()), 1)

    def testNewFolderCriterias(self):
        """
        testes that the import_criterias for a fold changed after
        being modifieded (folder bg)
        """
        duplicates_criteria = {'BookReference' : PersistentList(('authors', 'title')) }
  
        self.second_bf = self.getEmptyBibFolder('second_bib_folder')

        old_criterias = self.bf.getCriterias()
        sec_old_criterias = self.second_bf.getCriterias()
        
        for bib_type in duplicates_criteria.keys():
            self.bf.setCriteriasForType(bib_type, duplicates_criteria[bib_type])
 
        new_criterias = self.bf.getCriterias()
        sec_new_criterias = self.second_bf.getCriterias()

        self.assertNotEquals(old_criterias, new_criterias,
                              'Criterias have not been changed')
 
        self.assertEquals(sec_old_criterias, sec_new_criterias,
                              'Criterias have been changed')

    def testLocalGlobalDistinction(self):
        bibtool = self.portal.portal_bibliography
        filter = {'portal_type': bibtool.getReferenceTypes(), }

        # insert 3 entries in second_bib_folder
        second_bf = self.getEmptyBibFolder('second_bib_folder')
        bib_source = open(setup.BIBTEX_TEST_BIB, 'r').read()
        second_bf.processImport(bib_source, 'bibtex_test.bib')

        folder_contents = [ c for c in self.folder.contentIds() if c != '.personal' ]
        self.assertEquals(folder_contents, ['bib_folder', 'second_bib_folder'])
        self.assertEquals(len(folder_contents), 2)

        # do a global import of bib_folder
        bibtool = self.portal.portal_bibliography
        bib_source = open(setup.BIBTEX_TEST_BIB_DUP, 'r').read()
        entries = bibtool.getEntries(bib_source, None, 'bibtex_test_duplicates.bib')
        for entry in entries:
            if entry.get('title'):
                upload = self.bf.processSingleImport(entry, span_of_search='global')

        # check matches 2 
        dup_entry = self.bf.getDuplicatesFolder().contentValues()[0]
        self.assertEquals(len(dup_entry.getIs_duplicate_of()), 2)


    def testCriteriasScope(self):
        """
        testes
        set 1 :
           Changing criterias for a folder (self.bg) doesn't imply changing
           those of another folder (self.second_bg)
        set 2 :
           If criterias for a folder had been changed, changing
           portal criterias has no influence on them (self.bg),
           otherwise, a folder gets new portal criterias (self.second_bg)
        
        
        """
        bibtool = self.portal.portal_bibliography
        port_old_criterias = bibtool.getSelectedCriteria()

        folder_criterias = {'BookReference' : ('authors', 'title') }
        portal_criterias = {'ArticleReference' :
                                   ('publication_year', 'title')}
        self.second_bf = self.getEmptyBibFolder('second_bib_folder')

        #port_old_criterias = PersistentMapping()
        for key, value in bibtool.getSelectedCriteria().items():
            port_old_criterias[key] = value

        ### set 1
  
        old_criterias = self.bf.getCriterias()
        sec_old_criterias = self.second_bf.getCriterias()
        #changes criterias on first folder
        for bib_type in folder_criterias.keys():
            self.bf.setCriteriasForType(bib_type, \
                                        folder_criterias[bib_type])
        new_criterias = self.bf.getCriterias()
        sec_new_criterias = self.second_bf.getCriterias()

        #Criterias on Folder1 should got changed
        self.assertNotEquals(old_criterias, new_criterias,
                              'Folder1 criterias have not been changed')

        #Criterias for Folder2 should stay unchanged
        self.assertEquals(sec_old_criterias, sec_new_criterias,
                              'Folder2 criterias have been changed')
        
        #Criterias for Folder2 should be equal to those of Portal
	self.assertEquals(sec_new_criterias, port_old_criterias,
                              'Folder2 criterias are not equal to those of Portal')

        #Criterias for Folder1 should not be equal to those of Portal
        self.assertNotEquals(new_criterias, port_old_criterias,
                              'Folder1 criterias are equal to those of Portal')

        ### set 2
   
        #changes criterias on bibliography tool
        for bib_type in portal_criterias.keys():
            bibtool.setCriteriasForType(bib_type,\
                                        portal_criterias[bib_type])
        port_new_criterias = bibtool.getSelectedCriteria()
   
        new2_criterias = self.bf.getCriterias()
	
        self.second_bf = self.getEmptyBibFolder('another_second_bib_folder')
        sec_new2_criterias = self.second_bf.getCriterias()
  
        #portal criterias should got changed
        self.assertNotEquals(port_old_criterias, port_new_criterias,
                             'Portal criterias have not been changed')


        #Criterias for Folder1 should stay unchanged
        self.assertEquals(new_criterias, new2_criterias,
                              'Folder 1 criterias have been changed')
        #Criterias for Folder2 should be those of Portal
        self.assertEquals(port_new_criterias, sec_new2_criterias,
                              'Criterias For Folder 2 are not those of Portal')

    def testZeroCriterias(self):
        """
        testes an import without criterais specified
        """
	bibtool = self.portal.portal_bibliography
        filter = {'portal_type': bibtool.getReferenceTypes(), }

        self.bf.setCriterias({})
        self.insertDuplicates()
	# 4 bibref items, no duplicate folder
	self.assertEquals(len(self.bf.contentIds(filter=filter)), 4,
                          'Import failed, expected 4 entries')

    def testDuplicateImportFolderWideCriterias(self):
        """
        bibtex_test.bib contains 2 entries of BookReference type and
        1 entry of WebpublishedReference type
        """
        folder_criterias = {'BookReference' : () }
        bibtool = self.portal.portal_bibliography
        filter = {'portal_type': bibtool.getReferenceTypes(), }

        self.insertSameImportFile()
        self.assertEquals(len(self.bf.contentIds(filter=filter)), 3 ,
                          'Import failed, expected 3 entries')

        #set criterais for BookReference type to ()
        for bib_type in folder_criterias.keys():
            self.bf.setCriteriasForType(bib_type, \
                                        folder_criterias[bib_type])
        self.insertSameImportFile()
        self.assertEquals(len(self.bf.contentIds(filter=filter)), 5,
                          'Import failed, expected 5 entries')

    def testDuplicateImportCriterias(self):
        """
        bibtex_test.bib contains 2 entries of BookReference type and
        1 entry of WebpublishedReference type
        """
        portal_criterias = {'BookReference' : () }
        bibtool = self.portal.portal_bibliography
        filter = {'portal_type': bibtool.getReferenceTypes(), }

	# tests : portal wide criterias
        # set portal criterias for BookReference type to ()
        for bib_type in portal_criterias.keys():
            bibtool.setCriteriasForType(bib_type, \
                                        portal_criterias[bib_type])

        self.insertSameImportFile()
        self.assertEquals(len(self.bf.contentIds(filter=filter)), 3 ,
                          'Import failed, expected 3 entries')
        
	###
	### the next test is deprecated as the behaviour of the duplicates
	### engine has slightly changed: bib_tool duplicates criteria are
	### only adopted on bibfolder creation, afterwards there will be 
	### no further influence on bibfolder criteria by the bib_tool.
	### mg, 20061121
	
	## tests : portal wide criterias
        ## set portal criterias for BookReference type to ()
        #for bib_type in portal_criterias.keys():
        #    bibtool.setCriteriasForType(bib_type, \
        #                                portal_criterias[bib_type])
        #
	#self.insertSameImportFile()
	#print len(self.bf.contentIds(filter=filter))
        #self.assertEquals(len(self.bf.contentIds(filter=filter)), 5, 
        #                  'Import failed, expected 5 entries')

        # tests : folder wide criterias
        #set folder criterias for BookReference to ()
        
	folder_criterias = {'BookReference' : () }
        for bib_type in folder_criterias.keys():
            self.bf.setCriteriasForType(bib_type, \
                                        folder_criterias[bib_type])
        self.insertSameImportFile()
        self.assertEquals(len(self.bf.contentIds(filter=filter)), 5,
                          'Import failed, expected 5 entries')



    def testBibtexImportDuplicate_2matches(self):
        """ test that the references in the duplicated entry is multiple"""
        bibtool = self.portal.portal_bibliography
        filter = {'portal_type': bibtool.getReferenceTypes(), }

        bib_source = open(setup.BIBTEX_TEST_BIB_DUP, 'r').read()
        self.bf.processImport(bib_source, 'bibtex_test_duplicates.bib')
        self.assertEquals(len(self.bf.contentIds(filter=filter)), 3 ,
                          'Duplicate Entry was introduced, expected 3 unique entries')

    def testDelay(self):
        func_name = 'delayDuplicated'
        nb_end = 1
        expected_logs = []
        self.action_test(func_name, expected_logs, nb_end)

    def testSkip(self):
        func_name = 'skipDuplicated'
        nb_end = 0
        expected_logs = []
        self.action_test(func_name, expected_logs, nb_end)

    # FIXME - freeze - from previous (force)?
    def testReplace(self):
        func_name = 'replaceDuplicated'
        nb_end = 0
        expected_logs = ['replace begin implemented']
        self.action_test(func_name, expected_logs, nb_end)
        # check UID hasn't changed
        # check edition has changed to 3rd
        #self.bf. check 

    def testForce(self):
        bibtool = self.portal.portal_bibliography
        filter = {'portal_type': bibtool.getReferenceTypes(), }

        func_name = 'forceDuplicated'
        nb_end = 0
        expected_logs = None
        self.action_test(func_name, expected_logs, nb_end)
        self.assertEquals(len(self.bf.contentIds(filter=filter)), 4,
                          'Duplicate Entry was not forcefully introduced, expected 4 entries')

    def action_test(self, func_name, expected_logs, nb_end):
        """
        if expected_logs is None - don't test logs
        """
        logs = []
        self.insertDuplicates()
        duplicates = self.bf.getDuplicates()
        self.assertEquals(len(duplicates), 1, 'beginning count is not right')
        
        for key, each_duplicated_entry in duplicates.items():
            action_func = getattr(self.bf, func_name)
            msg = action_func(key)
            if msg:
                logs.append(msg)
        if not expected_logs is None:
            self.assertEquals(logs, expected_logs)

        duplicates = self.bf.getDuplicates()
        #self.assertEquals(len(self.bf._duplicates), nb_end)
        self.assertEquals(len(duplicates), nb_end, 'end count is not right')


    # test logs
    def testBibtexImportLogs(self):
        # do a global import of bib_folder
        bibtool = self.portal.portal_bibliography
        bib_source = open(setup.BIBTEX_TEST_BIB_DUP, 'r').read()
        entries = bibtool.getEntries(bib_source, None, 'bibtex_test_duplicates.bib')
        logs = []
        for entry in entries:
            if entry.get('title'):
                upload = self.bf.processSingleImport(entry, span_of_search='global')
		obj = upload[2]
                if upload:
                    logs.append(upload)
        self.assertEquals(logs, [('Skipped: Programming Python.\n', 'SKIPPED', obj, )])


  
        
def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestBibliographyFolder))
    return suite

if __name__ == '__main__':
    framework()



## Script (Python) "getPublicationsByAuthor"
##bind container=container
##bind context=context
##bind namespace=
##bind script=script
##bind subpath=traverse_subpath
##parameters=id=None
##title=
##
if id is None:
   member = context.portal_membership.getAuthenticatedMember()
else:
   member = context.portal_membership.getMemberById(id)
return member and member.getBRefs('authorOf') or []



from  bibtex import  BibtexRenderer, manage_addBibtexRenderer
from  pdf import  PDFRenderer, manage_addPDFRenderer
from  xml import  XMLRenderer, manage_addXMLRenderer
from  endnote import  EndRenderer, manage_addEndRenderer
from  ris import  RISRenderer, manage_addRISRenderer
# from  MedlineRenderer import  MedlineRenderer, manage_addMedlineRenderer

def initialize(context):
    context.registerClass(BibtexRenderer,
                          constructors = (manage_addBibtexRenderer,),
                          ) 
    context.registerClass(RISRenderer,
                          constructors = (manage_addRISRenderer,),
                          )
    context.registerClass(EndRenderer,
                          constructors = (manage_addEndRenderer,),
                          ) 
    context.registerClass(XMLRenderer,
                          constructors = (manage_addXMLRenderer,),
                          ) 
    context.registerClass(PDFRenderer,
                          constructors = (manage_addPDFRenderer,),
                          ) 
##     context.registerClass(MedlineRenderer,
##                           constructors = (manage_addMedlineRenderer,),
##                           ) 



# Zope imports
from persistent.mapping import PersistentMapping

# CMF imports
from Products.CMFCore.utils import getToolByName

# Archetypes import
from Products.Archetypes.public import listTypes
from Products.Archetypes.utils import shasattr

# Product imports
from Products.CMFBibliographyAT.config import REFERENCE_TYPES
from Products.CMFBibliographyAT.config import FOLDER_TYPES as BIBFOLDER_TYPES
from Products.CMFBibliographyAT.tool.parsers.citationmanager import \
     CitationManagerParser
     
from Products.CMFBibliographyAT.utils import _encode, _decode

import copy

# test the following bibref item schema attributes to find out if the schema needs a migration
CMFBAT_SCHEMA_UPGRADE_INDICATORS = [

    'annote',

]

class Migration(object):
    """Migrate from 0.8 to 0.9

    It *must* be safe to use this multiple times as it is run automatically
    upon (re)install in the portal_quickinstaller.
    """

    def __init__(self, site, out):
        self.site = site
        self.out = out
        #self.catalog = getToolByName(self.site, 'portal_catalog')

    def migrate(self):
        """Run migration on site object passed to __init__.
        """
        print >> self.out
	print >> self.out, u"Migrating CMFBibliographyAT 0.8 -> 0.9"
        bibtool = getToolByName(self.site, 'portal_bibliography')
        self.migrateTool(bibtool)
	if self.needsCriteriasManagerUpgrade():
	    self.migrateCriteriasManagerReferenceType()
	if self.needsTypeFieldSchemaUpgrade():
	    self.migrateTypeFieldSchema()
	if self.needsGeneralSchemaUpgrade():
	    self.generalSchemaUpgrade()
	if self.needsDuplicationEngineUpgrade():
	    self.migrateDuplicationEngine()

    def migrateTool(self, bibtool):
        """Migrate the bibtool.
        """
        # Check for and add a persistent dictionary to keep track of
        # registered reference types on the tool.
    	print >> self.out, u'BibliographyTool migration:'
    	print >> self.out, u'---------------------------'
        rt = getattr(bibtool, '_reference_types', None)
        if rt is None:
            msg = u"    Adding the tool's _reference_types attribute."
            print >> self.out, msg
            bibtool._reference_types = PersistentMapping()
            # By default, we just use the standard CMFBib ref types
            for ref_type in REFERENCE_TYPES:
                bibtool._reference_types[ref_type] = None
	else:
	    print >> self.out, u'    Tool is up-to-date'	
	print >> self.out    

    def needsCriteriasManagerUpgrade(self):
    
	"""Check for old publication_type (alias for meta_type) Criterias in BibFolders' CriteriasManager
	   -> should be renamed to "reference_type"
	"""
        print >> self.out, u'ImportCriteriasManager schema migration:'
        print >> self.out, u'----------------------------------------'	
	ct = getToolByName(self.site, 'portal_catalog')
	bibtool = getToolByName(self.site, 'portal_bibliography')

	# only check ref_types without typeField (id: publication_type)
	# if other ref_types have publication_type in ImportCriteriasManager 
	# then we definitely need to upgrade!!!
	ReferenceClasses_without_typeField = [ t['klass'] for t in listTypes() if (t['meta_type'] in bibtool.getReferenceTypes()) and ('publication_type' not in [ field.getName() for field in t['schema'].fields() ]) ]
	ReferenceTypes_without_typeField = tuple([ klass.meta_type for klass in ReferenceClasses_without_typeField ])

	count = 0
	brains = ct(portal_type=BIBFOLDER_TYPES, Language='all')
	for brain in brains[:5]:
	    bibfolder = brain.getObject()
	    
	    # if we come from a non-ImportCriteriasManager version of CMFBAT, we have to be carefull here.
	    if shasattr(bibfolder, 'imp_criterias'):
		sel_crits = bibfolder.getSelectedCriterias()
		for ref_type in sel_crits.keys():
	    	    if (ref_type in ReferenceTypes_without_typeField) and ('publication_type' in sel_crits[ref_type]):
			count += 1

	if count:
	    print >> self.out, u'    Upgrade of ImportCriteriasManager required.'
	    return True

	print >> self.out, u'    No ImportCriteriasManager upgrade needed.'
	print >> self.out
        return False
    
    def	migrateCriteriasManagerReferenceType(self):
    
	ct = getToolByName(self.site, 'portal_catalog')
	bibtool = getToolByName(self.site, 'portal_bibliography')
	brains = ct(portal_type=BIBFOLDER_TYPES, Language='all')

	ReferenceClasses_without_typeField = [ t['klass'] for t in listTypes() if (t['meta_type'] in bibtool.getReferenceTypes()) and ('publication_type' not in [ field.getName() for field in t['schema'].fields() ]) ]
	ReferenceTypes_without_typeField = tuple([ klass.meta_type for klass in ReferenceClasses_without_typeField ])

	for brain in brains:
	    bibfolder = brain.getObject()
	    
	    # reinitialize a bibfolder's possible import criterias for each reference type
	    bibfolder.initCriterias()
	    print >> self.out, u'    Migrating import criteria for BibliographyFolder: %s' % (bibfolder.getId())
	    sel_crits = copy.deepcopy(bibfolder.getSelectedCriterias())

	    all_crits = {}
	    for ref_type, crits in copy.deepcopy(bibfolder.allCriterias()):
		all_crits[ref_type] = crits

	    for ref_type in sel_crits.keys():
		
		# publication_type always migrate for classes with no typeField
		# and if there is only one publication_type in the list
		if ref_type in ReferenceTypes_without_typeField:
    		    if 'publication_type' in sel_crits[ref_type]:
			del sel_crits[ref_type][sel_crits[ref_type].index('publication_type')]
			if ('reference_type' not in sel_crits[ref_type]) and ('reference_type' in all_crits[ref_type]):
			    sel_crits[ref_type].append('reference_type')
		
		# classes with typeField: migrate only if old type value still in dict
		# or if publication_type appears twice in the list.
		if ref_type not in ReferenceTypes_without_typeField:
		    
		    # if the refitem schema is already migrated (who knows why...) we have to seek
		    # for criteria lists that have two publication_type entries
		    more_than_one_publication_type = ( 1 < len([ field for field in bibfolder.allCriterias() if field == 'publication_type' ]) )
    		
		    if ('type' in sel_crits[ref_type]) or more_than_one_publication_type:
			
			if 'publication_type' in sel_crits[ref_type]:

			    # this will delete only one publiation_type entry
			    del sel_crits[ref_type][sel_crits[ref_type].index('publication_type')]
			    if ('reference_type' not in sel_crits[ref_type]) and ('reference_type' in all_crits[ref_type]):
				sel_crits[ref_type].append('reference_type')
    			
			if 'type' in sel_crits[ref_type]:
			    del sel_crits[ref_type][sel_crits[ref_type].index('type')]
			    if ('publication_type' not in sel_crits[ref_type]) and ('publication_type' in all_crits[ref_type]):
				sel_crits[ref_type].append('publication_type')		
				
		    else:		

			if 'publication_type' in sel_crits[ref_type]:
			    del sel_crits[ref_type][sel_crits[ref_type].index('publication_type')]
			    if ('reference_type' not in sel_crits[ref_type]) and ('publication_type' in all_crits[ref_type]):
				sel_crits[ref_type].append('reference_type')
    				
	    #print 'Migrating criterias of BibFolder: %s' % bibfolder.getId()
	    #print sel_crits
	    bibfolder.setCriterias(sel_crits)
	print >> self.out    
	    	
    def needsTypeFieldSchemaUpgrade(self):
	"""Check for old straying type field remnants, they indicate a typeField migration need
	"""
        print >> self.out, u'typeField schema migration:'
        print >> self.out, u'---------------------------'	
	ct = getToolByName(self.site, 'portal_catalog')
	bibtool = getToolByName(self.site, 'portal_bibliography')

	# detect reference_types that contain the typeField
	ReferenceClasses_with_typeField = [ t['klass'] for t in listTypes() if (t['meta_type'] in bibtool.getReferenceTypes()) and ('publication_type' in [ field.getName() for field in t['schema'].fields() ]) ]
	ReferenceTypes_with_typeField = tuple([ klass.meta_type for klass in ReferenceClasses_with_typeField ])

	brains = ct(portal_type=ReferenceTypes_with_typeField, Language='all')
	# check ALL(!) brains if we really need schema upgrade for old typeField: 'type' -> 'publication_type'
	for brain in brains:
	    old_typeField_value = getattr(brain.getObject(), 'type', False)

	    # to avoid mismatch with the BaseObject's type definition (alias for schema) check 
	    # for a unicode value in typeField
	    if old_typeField_value and (type(old_typeField_value) == type(u'')):
        	print >> self.out, u'    Upgrade of typeField required.'
		return True

        print >> self.out, u'    No typeField schema upgrade needed.'
	print >> self.out
	return False
	
    def migrateTypeFieldSchema(self):
	"""Migrate typeField id 'type' to new (less overloaded) id 'publication_type'
	"""
	ct = getToolByName(self.site, 'portal_catalog')
	bibtool = getToolByName(self.site, 'portal_bibliography')

	# detect reference_types that contain the typeField
	ReferenceClasses_with_typeField = [ t['klass'] for t in listTypes() if (t['meta_type'] in bibtool.getReferenceTypes()) and ('publication_type' in [ field.getName() for field in t['schema'].fields() ]) ]
	ReferenceTypes_with_typeField = tuple([ klass.meta_type for klass in ReferenceClasses_with_typeField ])

	brains = ct(meta_type=ReferenceTypes_with_typeField, Language='all')
	for brain in brains:
	    
	    bibref_item = brain.getObject()
	    # this one is the old typeField value
	    old_typeField_value = copy.deepcopy(getattr(bibref_item, 'type', False))
	    # this one the new typeField with id 'publication_type'
	    new_typeField = bibref_item.Schema().get('publication_type', None)

	    # if there is no formerly set typeField value, just do the schema update
	    if new_typeField and (type(old_typeField_value) == type(bibref_item.schema)):
		bibref_item._updateSchema()
		try:
            	    print >> self.out, u'    Only simple schema update needed for ObjectId %s: typeField value=\'%s\'' % (brain.getId, bibref_item.getPublication_type())
		except UnicodeDecodeError:
        	    print >> self.out, u'    Only simple schema update needed for ObjectId %s: typeField value=\'%s\'' % (brain.getId, '<HIDDEN: value contains non-ASCII characters>')
	    
	    # but if there is an old 'type' attribute in the bibref_item object, update the new typeField with its value
	    # to avoid mismatch with the BaseObject's type definition (alias for schema) check for a unicode value in typeField
	    elif new_typeField and (type(old_typeField_value) == type(u'')):
		delattr(bibref_item, 'type')
		bibref_item._updateSchema()
		bibref_item.edit(publication_type=old_typeField_value)
		try:
            	    print >> self.out, u'    Migrating typeField of ObjectId %s: value=\'%s\'' % (brain.getId, bibref_item.getPublication_type())
		except UnicodeDecodeError:
        	    print >> self.out, u'    Migrating typeField of ObjectId %s: value=\'%s\'' % (brain.getId, '&lt;HIDDEN: value contains non-ASCII characters&gt;')

	    else:
		
		print >> self.out, u'    typeField schema update for ObjectId: %s failed.' % bibref_item.getId()
	
	print >> self.out	     		

    def needsGeneralSchemaUpgrade(self):
        """Returns True if one of the first 5 bibitems found
	   has missing schema fields (compared to v0.8); called
	   by the installer to figure out whether a schema update
	   is needed."""
	print >> self.out, u"general schema upgrade of bibliographical reference items"
	print >> self.out, u"---------------------------------------------------------"
	ct = getToolByName(self.site, 'portal_catalog')
	bib_tool = getToolByName(self.site, 'portal_bibliography')
	brains = ct(portal_type=bib_tool.getReferenceTypes(), Language='all')
	
	# needs schema upgrade for authors field
	if brains:
	    for attribute in CMFBAT_SCHEMA_UPGRADE_INDICATORS:
		for brain in brains[:10]:
		    if not shasattr(brain.getObject(), attribute, False):
			return True
	print >> self.out, u"    No general schema upgrade needed."
	print >> self.out
	return False

    # migrate data from old to new schema
    def generalSchemaUpgrade(self):
        """perform a general AT schema upgrade"""
	ct = getToolByName(self.site, 'portal_catalog')
	bib_tool = getToolByName(self.site, 'portal_bibliography')

	# logging to ZLog and to quick installer's report 
        print >> self.out, u'    Bibliographical references need general schema upgrade!!! This might take a while...'
        print u'***'
	print u'*** CMFBibliogaphyAT migration: Bibliographical references need general schema upgrade!!! This might take a while...'
	print u'***'
	brains = ct(portal_type=bib_tool.getReferenceTypes(), Language='all')
	for brain in brains:
	    obj = brain.getObject()
	    obj._updateSchema()

	print u'CMFBibliographyAT migration: Upgraded schemata of %s bibliographical references' % len(brains)
	print 
	print >> self.out, u'    Upgraded schemata of %s bibliographical references' % len(brains)
	print >> self.out 											    

    def needsDuplicationEngineUpgrade(self):
        """Returns True if any existing bibfolder shows signs of bibfolder._duplicates """
	print >> self.out, u"upgrade of bibliography folders' duplicate bibliographical reference engine" 
	print >> self.out, u"---------------------------------------------------------------------------"
	ct = getToolByName(self.site, 'portal_catalog')
	bib_tool = getToolByName(self.site, 'portal_bibliography')
	brains = ct(portal_type=BIBFOLDER_TYPES, Language='all')
	
	# needs schema upgrade for authors field
	if brains:
	    for brain in brains:
		if shasattr(brain.getObject(), '_duplicates'):
			return True
	print >> self.out, u"    No duplication engine migration needed."
	print >> self.out
	return False

    # migrate duplication engine
    def migrateDuplicationEngine(self):
	""" migrate bibfolder._duplicates -> bibfolder.duplicates.<items> """
	ct = getToolByName(self.site, 'portal_catalog')
	bib_tool = getToolByName(self.site, 'portal_bibliography')
	at_tool = getToolByName(self.site, 'archetype_tool')

        print >> self.out, u'    Bibliography Folder needs duplication engine upgrade ...'
	brains = ct(portal_type=BIBFOLDER_TYPES, Language='all')
	for brain in brains:
	    bibfolder = brain.getObject()
	    if shasattr(bibfolder, '_duplicates'):
		duplicates = bibfolder._duplicates
		for key in duplicates.keys():
		    entry = duplicates[key]
		    if entry.has_key('publication_type') and entry['publication_type'] in bib_tool.getReferenceTypes():
			# very old instance...
			entry['reference_type'] = entry['publication_type']
			del entry['publication_type']
			if entry.has_key('type'):
			    entry['publication_type'] = entry['type']

		    matched_uuids = entry['matched_uuids']
		    del entry['matched_uuids']
		    (report_line, import_status, bibref_item) = bibfolder.processSingleImport(entry, force_to_duplicates=True)
		    print >> self.out, '    Transferred _duplicates item %s to object %s in %s/duplicates' % (key, bibref_item.getId(), bibfolder.absolute_url())
		    bibref_item.setIs_duplicate_of([ matched_uuid for matched_uuid in matched_uuids if at_tool.lookupObject(matched_uuid) ])
		delattr(bibfolder, '_duplicates')    
		    
	print >> self.out, u'    Upgraded of bibliography folders\' duplication engine completed. %s bibliography folders migrated.' % len(brains)
	print >> self.out 											    



import os
from zLOG import LOG, PROBLEM
from Products.CMFCore.permissions import AddPortalContent

try:
    from Products.LinguaPlone.public import registerType
    CMFBAT_USES_LINGUAPLONE=True
except:    
    CMFBAT_USES_LINGUAPLONE=False
# manually disable here
CMFBAT_USES_LINGUAPLONE=False    

ADD_CONTENT_PERMISSION = AddPortalContent
PROJECTNAME = "CMFBibliographyAT"
SKINS_DIR = 'skins'

USE_EXTERNAL_STORAGE = True
EXTERNAL_STORAGE_PATH = 'files'

GLOBALS = globals()
ENTRY_TYPES = 'bibtex_types'
FOLDER_TYPES = ('BibliographyFolder',
                'LargeBibliographyFolder',
                )
PREPRINT_SERVERS = ('None:',
                    'arXiv:http://arxiv.org',
                    'CogPrints:http://cogprints.ecs.soton.ac.uk/',
                    )

## Old way to define possible bibliographical content types
#types = os.listdir(os.path.dirname(__file__)+'/'+ ENTRY_TYPES)
#types = [type[:-3] for type in types \
#          if (type.endswith('.py') \
#              and not type.startswith('__'))]
#REFERENCE_TYPES = tuple(types)

## New way (since product reorganization)
REFERENCE_TYPES = (
    'ArticleReference', 
    'BookletReference', 
    'BookReference', 
    'ConferenceReference', 
    'InbookReference', 
    'IncollectionReference', 
    'InproceedingsReference', 
    'ManualReference', 
    'MastersthesisReference', 
    'MiscReference', 
    'PhdthesisReference', 
    'PreprintReference', 
    'ProceedingsReference', 
    'TechreportReference', 
    'UnpublishedReference', 
    'WebpublishedReference',
    )

ZOPE_TEXTINDEXES = (
    'TextIndex',
    'ZCTextIndex',
    'TextIndexNG2',
    'TextIndexNG3',
)
if USE_EXTERNAL_STORAGE:
    try:
        import Products.ExternalStorage
    except ImportError:
        LOG('CMFBibliographyAT',
            PROBLEM, 'ExternalStorage N/A, falling back to AnnotationStorage')
        USE_EXTERNAL_STORAGE = False



############################################################################
#                                                                          #
#             copyright (c) 2003 ITB, Humboldt-University Berlin           #
#             written by: Raphael Ritz, r.ritz@biologie.hu-berlin.de       #
#                                                                          #
############################################################################

"""BibtexParser class"""

# Python stuff
import re

# Zope stuff
from Globals import InitializeClass
from App.Dialogs import MessageDialog

# Bibliography stuff
from Products.CMFBibliographyAT.tool.parsers.base \
     import IBibliographyParser, BibliographyParser

from Products.CMFBibliographyAT.utils import _encode, _decode
from Products.CMFBibliographyAT.encodings import _latex2utf8enc_mapping

# our custom entity converter
from entities import Convert
convert = Convert()

_encoding = 'utf-8'   # XXX: should be taken from the site configuration

class BibtexParser(BibliographyParser):
    """
    A specific parser to process input in BiBTeX-format.
    """

    __implements__ = (IBibliographyParser ,)

    meta_type = "Bibtex Parser"

    format = {'name':'BibTeX',
              'extension':'bib'}

    def __init__(self,
                 id = 'bibtex',
                 title = "BibTeX parser",
                 delimiter = '}\s*@',
                 pattern = '(,\s*\w{2,}\s*=)'):
        """
        initializes including the regular expression patterns
        """
        self.id = id
        self.title = title
        self.setDelimiter(delimiter)
        self.setPattern(pattern)


    # Here we need to provide 'checkFormat' and 'parseEntry'
    # FIXME TODO - refactor with pyblbibtex
    def checkFormat(self, source):
        """
        is this my format?
        """
        # vanilla test for 'author' or 'editor'
        # in the sub-string 'source[10, 100]'
        ## rr: can definitively be improved

        teststring = source[10:100].lower()
        ai = teststring.find('author')
        ei = teststring.find('editor')
        if ai + ei > -2:
            return 1
        else:
            return 0

    def preprocess(self, source):
        """
        expands LaTeX macros
        removes LaTeX commands and special formating
        converts special characters to their HTML equivalents
        """
        source = self.expandMacros(source)
        source = self.stripComments(source)
	source = self.convertChars(source)
	# it is important to convertChars before stripping off commands!!!
	# thus, whatever command will be replaced by a unicode value... the 
	# remaining LaTeX commands will vanish here...
        source = self.stripCommands(source)
        return source

    def expandMacros(self, source):
        source = self.expandStringMacros(source)
        # add more macro conventions here if there are any
        return source

    def expandStringMacros(self, source):
        lines = source.split('\n')
        macros = []
        sourcelns = []
        for line in lines:
            if line.find('@String') > -1:
                macros.append(line)
            else:
                sourcelns.append(line)
        source = '\n'.join(sourcelns)
        for macro in macros:
            split_on = re.compile('[{=}]+')
            raw_matches = split_on.split(macro)
            matches = [m for m in raw_matches if m not in ['', ' ', '\r']]
            # raise str(matches)
            short = matches[1].strip()
            long = matches[-1].strip()
            pattern = "\\b" + short + "\\b"
            old = re.compile(pattern)
            source = old.sub(long, source)
        return source
    
    def stripCommands(self, source):
        oldstyle_cmd = re.compile(r'{\\[a-zA-Z]{2,}')
        newstyle_cmd = re.compile(r'\\[a-zA-Z]+{')
        source = oldstyle_cmd.sub('{', source)
        source = newstyle_cmd.sub('{', source)
        return source

    def stripComments(self, source):

        inside_entry = False
        waiting_for_first_brace = False
        newsource = ''

        for idx in range(len(source)):
    
            char = source[idx]
            last_char = (idx > 0) and source[idx-1] or '\n'
            next_char = (idx < len(source)-1) and source[idx+1] or '\n'

            if char == '@' and not inside_entry:
                inside_entry = True
                waiting_for_first_brace = True
                braces_nesting_level = 0
        
            if inside_entry:
        
                newsource = newsource + char
                if char == '{' and last_char != "\\":
                    braces_nesting_level += 1

                if char == '}' and last_char != "\\":
                    braces_nesting_level -= 1

                if waiting_for_first_brace and (braces_nesting_level == 1):
                    waiting_for_first_brace = False

                if (braces_nesting_level == 0) and not waiting_for_first_brace and (char == '}'):
                    inside_entry = False
                    newsource = newsource + "\n"

        completely = re.compile('.*')
        ## this line caused issue #20, leaving it here for now... mg-20061023
	# source = completely.sub(newsource, 'dummy')
	## this line fixes issue #20
	source = newsource
        return source

    def convertChars(self, source):
        source = self.convertLaTeX2Unicode(source)
        source = self.fixWhiteSpace(source)
        return self.explicitReplacements(source)

    def convertLaTeX2Unicode(self, source):

	for latex_entity in _latex2utf8enc_mapping.keys():
	    source = _encode(_decode(source).replace(latex_entity, _latex2utf8enc_mapping[latex_entity]))
    
        return source

    def uml2h(self, hit):
        return convert(r'&' + hit.group(1) + 'uml;').encode(_encoding)
    def ac2h(self, hit):
        return convert(r'&' + hit.group(1) + 'acute;').encode(_encoding)
    def gr2h(self, hit):
        return convert(r'&' + hit.group(1) + 'grave;').encode(_encoding)
    def cf2h(self, hit):
        return convert(r'&' + hit.group(1) + 'circ;').encode(_encoding)
    def tilde2h(self, hit):
        return convert(r'&' + hit.group(1) + 'tilde;').encode(_encoding)

    def fixWhiteSpace(self, source):
        ttable = [(r'\ ', ' '),
                  (r'\!', ' '),
                  ]
        source = self.mreplace(source, ttable)
        wsp_tilde = re.compile(r'[^/\\]~')
        return wsp_tilde.sub(self.tilde2wsp, source).replace('\~', '~')

    def tilde2wsp(self, hit): return hit.group(0)[0] + ' '

    def explicitReplacements(self, source):
        # list of 2 tuples; second element replaces first
        ttable = [(r'\/', ''),
                  (r'\&', '&'),
                  (r'\~', '~'),
                  (r'---', '&mdash;'),
                  (r'--', '&ndash;'),
                  ]
        return self.mreplace(source, ttable)
        
    def mreplace(self, s, ttable):
        for a, b in ttable:
            s = s.replace(a, b)
        return s
    
    # done with preprocessing

    def parseEntry(self, entry):
        """
        parses a single entry
        
        returns a dictionary to be passed to
        BibliographyEntry's edit method
        """
        result = {}
        authorlist = []
        authorURLlist = []
        
        # remove newlines and <CR>s, and remove the last '}'
        entry = entry.replace('\n', ' ').replace('\r', '').replace('\t', ' ').rstrip().rstrip('}')
        tokens = self.pattern.split(entry)
    
        try: 
            type, pid = tokens[0].strip().split('{')
            type = type.replace('@', '').strip().lower()
            result['reference_type'] = type.capitalize() + 'Reference'
            result['pid'] = pid.replace(',', '').strip()
        except:
            return "Bibtex Parser Error: malformed first line."

        for k,v in self.group(tokens[1:],2):
            key = k[1:-1].strip().lower()
            # INBOOKs mapping: title -> booktitle, chapter -> chapter and title
            if type == 'inbook':
                if key == 'title':
                    key = 'booktitle'

                if key == 'chapter':
                    result['title'] = self.clean(v.rstrip().rstrip(',').rstrip())
            
            # BibTex field "type" maps to CMFBAT field "publication_type"
            if key == 'type':
                key = 'publication_type'
                result[key] = self.clean(v.rstrip().rstrip(',').rstrip())
            
            # special procedure for authors and editors         
            elif key == 'author':
                if result.has_key('author'):
                    result[key].append( self.clean(v.rstrip().rstrip(',').rstrip()) )
                else:
                    result[key] = [ self.clean(v.rstrip().rstrip(',').rstrip()), ]
            elif (key == 'editor') and (type in ['book','proceedings']):
                if result.has_key('editor'):
                    result[key].append( self.clean(v.rstrip().rstrip(',').rstrip()) )
                else:
                    result[key] = [ self.clean(v.rstrip().rstrip(',').rstrip()), ]
            else:
                value = self.clean(v.rstrip().rstrip(',').rstrip())
                result[key] = value
                # Aliasing the value to an upper-cased key so that when this dictionary
                # is passed into <a_reference_object>.edit(**dictionary), the values
                # will match and auto-update schema fields that are specified in either
                # upper or lower case.  This is motivated by the 'DOI' field being upper-cased.
                # Of course, this won't help mixed-case fields, but we'd probably need to change
                # Archetype internals to fix that - and that would be a questionable endeavour.
                result[key.upper()] = value
        
            #print key, result[key]

        # compile authors list of dictionaries
        # we can have authors   
        if result.has_key('author'):
            for each in result['author']:
                each = each.replace(' AND', ' and')
                authorlist.extend( each.split(' and') )
        # but for some bibref types we can have editors alternatively
        elif result.has_key('editor') and (type in ['book','proceedings']):
            result['editor_flag'] = True
            for each in result['editor']:
                each = each.replace(' AND', ' and')
                authorlist.extend( each.split(' and') )
        if result.has_key('authorURLs'):
            authorURLlist = result['authorURLs'].split('and ')

        if authorlist:
            alist = []
            authorlist = [x for x in authorlist if x]
            for author in authorlist:
                fname = mname = lname = ''
                parts = self.splitAuthor(author)
                if len(parts) == 1:
                    lname = parts[0].strip()
                else:
                    lname = parts[-1].strip()
                    fname = parts[0].strip()
                    if parts[1:-1]:
                        for part in parts[1:-1]:
                            mname = mname + part.strip()
                adict = {'firstname': fname,
                         'middlename': mname,
                         'lastname': lname}
                alist.append(adict)

        if authorURLlist and alist:
            index = 0
            for url in authorURLlist:
                alist[index]['homepage'] = url.strip()
                index += 1

        if authorlist:
            result['authors'] = alist

        # do some renaming and reformatting
        tmp = result.get('note')
        while tmp and tmp[-1] in ['}', ',', '\n', '\r']:
            tmp = tmp[:-1]
        if tmp:
            result['note'] = tmp
        # make keywords a list
        try:
            tmp = eval(result.get('keywords', '[]'))
        except:
            tmp = result.get('keywords').split(',')
        result['keywords'] = tmp
        result['publication_year'] = result.get('year', '')
        result['publication_month'] = result.get('month', '')
        result['publication_url'] = result.get('url', '')
        ## result['publication_title'] = result.get('title', '')
        tmp = result.get('title','')
        for car in ('\n', '\r', '\t'):
            tmp = tmp.replace(car, ' ')
        while '  ' in tmp:
            tmp = tmp.replace('  ', ' ')
        result['title'] = tmp

        return result

    # the helper method's

    def splitAuthor(self, author=None):
        if not author: return []
        #parts = author.replace('.', ' ').split(',',1)
        parts = author.split(',',1)
        if len(parts) == 1: return parts[0].split()
        else:
            tmp = parts[1].split()
            tmp.append(parts[0])
            return tmp

    def clean(self, value):
        value = value.replace('{', '').replace('}', '').strip()
        if value and value[0] == '"' and len(value) > 1:
            value = value[1:-1]
        return value

    def group(self, p,n):
        """ Group a sequence p into a list of n tuples."""
        mlen, lft = divmod(len(p), n)
        if lft != 0: mlen += 1

        # initialize a list of suitable length
        lst = [[None]*n for i in range(mlen)]
        
        # Loop over all items in the input sequence
        for i in range(len(p)):
            j,k = divmod(i,n)
            lst[j][k] = p[i]

        return map(tuple, lst)    
    
InitializeClass(BibtexParser)

   
def manage_addBibtexParser(self, REQUEST=None):
    """ """
    try:
        self._setObject('bibtex', BibtexParser())
    except:
        return MessageDialog(
            title='Bibliography tool warning message',
            message='The parser you attempted to add already exists.',
            action='manage_main')    
    return self.manage_main(self, REQUEST)



############################################################################
#                                                                          #
#             copyright (c) 2003 ITB, Humboldt-University Berlin           #
#             written by: Raphael Ritz, r.ritz@biologie.hu-berlin.de       #
#                                                                          #
############################################################################

"""CitationManager parser class"""

# Zope stuff
from Globals import InitializeClass
from App.Dialogs import MessageDialog

# Bibliography stuff
from Products.CMFBibliographyAT.tool.parsers.base \
     import IBibliographyParser, BibliographyParser
from Products.CMFBibliographyAT.tool.parsers.base \
     import EntryParseError

# Python standard library imports
import re
from cStringIO import StringIO
from types import StringType


# mapping from Citation Manager keys to CMBBib keys
key_mapping = {  'TI'   : 'title'
               , 'AB'   : 'abstract'
               #, 'GR' :
               #, 'FT' :
               #, 'AU' : Murphy, Kevin M.; Shleifer, Andrei; Vishny, Robert W.
               , 'SO' : 'journal'
               #, 'S2' :
               , 'VO' : 'volume'
               , 'NO' : 'number'
               #, 'SE' :
               #, 'DA' : May, 1991
               , 'PP' : 'pages'
               , 'EI' : 'publication_url'
               #, 'IN' : 00335533
               #, 'KW' :
               , 'PB' : 'publisher'
               #, 'LO' :
               #, 'LA' :
               #, 'CR' : Copyright 1991 The MIT Press
              }


class ParseException(Exception):
    pass


class CitationManagerParser(BibliographyParser):
    """
    A specific parser to process input in CitationManager format.
    """

    __implements__ = (IBibliographyParser ,)

    meta_type = "CitationManager Parser"

    format = {'name':'CitationManager',
              'extension':'txt'}

    def __init__(self,
                 id = 'citationmanager',
                 title = "CitationManager parser",
                 delimiter = '\n\n',
                 #pattern = r'(^.{0,4}- )',
                 flag = re.M):
        """
        initializes including the regular expression patterns
        """
        self.id = id
        self.title = title
        self.setDelimiter(delimiter)
        #self.setPattern(pattern, flag)


    # Here we need to provide 'checkFormat' and 'parseEntry'


    def preprocess(self, source):
	""" remove superfluous <CR>s
	"""
	return source.replace('\r','')

    def checkFormat(self, source):
        """
        is this my format?
        """
        #raise Exception(type(source))
        #if source.find('      AU: ') > -1:
        #    return 1
        #else:
        #    return 0
        return True

    def parseEntry(self, entry):
        """
        parses a single entry
        
        returns a dictionary to be passed to
        BibliographyEntry's edit method
        """
        if isinstance(entry, StringType):
            entry = StringIO(entry)
        key = ''
        val = []
        record = {}
        first_line = True
        new_field = True
        for line in entry.readlines():
            if line[0] == '<' or line[0].strip() == '':
                # we ignore the number markers e.g. '<1>' and extra empty
                # lines
                continue
            find_delimiter = line.find(' : ')
            if find_delimiter != -1:
                new_field = True
            if new_field and not first_line:
                record[key] = ' '.join(val)
                val = []
            if new_field:
                key, value = line.split(' : ')
                val.append( value.strip() )
            else:
                val.append( line.strip() )
                #record[key] = '\n'.join(val)
            first_line = False
        result = {}
        if record['IT'] != 'FLA':
            # FLA seems to mean article.  We don't know how to deal with
            # anything else.
            entry.seek(0)
            raise EntryParseError(entry.read().replace('\n', '\\n'))
        result['reference_type'] = 'ArticleReference'
        self._parseAU(record['AU'], result)
        self._parseDate(record['DA'], result)
        for key in key_mapping.keys():
            if record.has_key(key):
                result[key_mapping[key]] = record[key]
        return result

    def _parseAU(self, s, result):
        # e.g. 'Murphy, Kevin M.; Shleifer, Andrei; Vishny, Robert W.'
        d = []
        l = s.split('; ')
        for person in l:
            p = {}
            p['lastname'], forenames = person.split(', ')
            forenames = forenames.split(' ')
            p['firstname'] = forenames[0]
            if len(forenames) > 1:
                p['middlename'] = ' '.join(forenames[1:])
            else:
                p['middlename'] = ''
            d.append(p)
        result['authors'] = d

    def _parseDate(self, s, result):
        # e.g. "Mar., 1967"
        month, year = s.split(', ')
        result['publication_year'] = year
        result['publication_month'] = month


InitializeClass(CitationManagerParser)

   
def manage_addCitationManagerParser(self, REQUEST=None):
    """ """
    try:
        self._setObject('citationmanager', CitationManagerParser())
    except:
        return MessageDialog(
            title='Bibliography tool warning message',
            message='The parser you attempted to add already exists.',
            action='manage_main')
    return self.manage_main(self, REQUEST)



from Products.Archetypes.public import *
from Products.CMFCore import CMFCorePermissions

schema = BaseSchema

actions = (
    {
    'id' : 'view',
    'name' : 'View',
    'action' : 'folder_listing',
    'permissions' : (CMFCorePermissions.View, )
    },
    {
    'id' : 'edit',
    'name' : 'Edit',
    'action' : 'portal_form/base_edit',
    'permissions' : (CMFCorePermissions.ModifyPortalContent, )
    },
    {
    'id' : 'properties',
    'name' : 'Properties',
    'action' : 'portal_form/base_metadata',
    'permissions' : (CMFCorePermissions.ModifyPortalContent, )
    },
    {
    'id' : 'import',
    'name' : 'Import',
    'action' : 'portal_form/import',
    'permissions' : (CMFCorePermissions.ModifyPortalContent, )
    },
    )

class BibFolder(BaseFolder):
    """A folder to contain bibliographic objects"""
    schema = schema
    actions = actions
    include_default_actions = 0
    global_allow = 1
    filter_content_types = 1
    allowed_content_types = (['Book', 'Journal', 'Article', 'Chapter', 'Thesis', 'Proceedings', 'TechnicalReport', 'ConferencePaper', 'NewsArticle', 'Standard'])
    archetype_name = "Bibliographical Folder"

registerType(BibFolder)


############################################################################
#                                                                          #
#             copyright (c) 2004 ITB, Humboldt-University Berlin           #
#             written by: Raphael Ritz, r.ritz@biologie.hu-berlin.de       #
#                                                                          #
############################################################################

"""pybliographer's BibtexParser class"""

# Python stuff
import re, os
from types import StringTypes

# pybliographer stuff
HAS_PYBLIOGRAPHER = 1
try:
    import _bibtex
except ImportError:
    HAS_PYBLIOGRAPHER = 0

# Zope stuff
from Globals import InitializeClass
from App.Dialogs import MessageDialog

# Bibliography stuff
from Products.CMFBibliographyAT.tool.parsers.base \
     import IBibliographyParser, BibliographyParser


class PyBlBibtexParser(BibliographyParser):
    """
    A specific parser to process input in BiBTeX-format.
    """

    __implements__ = (IBibliographyParser ,)

    meta_type = "Pybliographers Bibtex Parser"

    format = {'name':'BibTeX (pybl)',
              'extension':'bib'}

    def __init__(self,
                 id = 'pyblbibtex',
                 title = "Pybliographers BibTeX parser",
                 delimiter = '(~\s*$)*',
                 pattern = '(,\s*\w{2,}\s*=)'):
        """
        initializes including the regular expression patterns
        """
        self.id = id
        self.title = title
        self.setDelimiter(delimiter)
        self.setPattern(pattern)


    # Here we need to provide 'checkFormat' and 'parseEntry'
    # FIXME TODO - refactor with bibtex
    def checkFormat(self, source):
        """
        is this my format?
        """
        # vanilla test for 'author' or 'editor'
        # in the sub-string 'source[10, 100]'
        ## rr: can definitively be improved

        teststring = source[10:100].lower()
        ai = teststring.find('author')
        ei = teststring.find('editor')
        if ai + ei > -2:
            return 1
        else:
            return 0

    def preprocess(self, source):
        """
        expands LaTeX macros
        """
        return self.expandMacros(source)

    def expandMacros(self, source):
        source = self.expandStringMacros(source)
        # add more macro conventions here if there are any
        return source

    def expandStringMacros(self, source):
        lines = source.split('\n')
        macros = []
        sourcelns = []
        for line in lines:
            if line.find('@String') > -1:
                macros.append(line)
            else:
                sourcelns.append(line)
        source = '\n'.join(sourcelns)
        for macro in macros:
            split_on = re.compile('[{=}]+')
            raw_matches = split_on.split(macro)
            matches = [m for m in raw_matches if m not in ['', ' ', '\r']]
            short = matches[1].strip()
            long = matches[-1].strip()
            pattern = "\\b" + short + "\\b"
            old = re.compile(pattern)
            source = old.sub(long, source)
        return source

    def getEntries(self, source):
        """
        main method to be called for parsing the entries
        in 'source' - a string representation of the source file
        """
        source = self.preprocess(source) # would pybliographer do that anyway?
        file = self.toBibIO(source)
        raw_entries = self.parsedEntries(file)
        return self.mapp_entries(raw_entries)

    def parsedEntries(self, file):
        """the main routine from pybliographer"""
        def expand (file, entry):
            items = entry [4]
            for k in items.keys ():
                items [k] = _bibtex.expand (file, items [k], -1)
            return
        results = []
        while 1:
            try:
                entry = _bibtex.next (file)
                if entry is None: break
                expand (file, entry)
                results.append(entry)
##                 obtained = `entry`
            except IOError, msg:
                obtained = 'ParserError'
        return results

    def toBibIO(self, source):
        """ugly hack to get a bibtex file object"""
        wdir = self._makeWorkingDirectory()
        return self._makeBibFile(wdir, source)
        
    def _makeWorkingDirectory(self, path=None):
        if not path:
            path = '/tmp/bib-tool'
        try:
            os.mkdir(path)
        except OSError:
            pass
        return path

    def _makeBibFile(self, work_dir, source):
        full_path = os.path.join(work_dir, 'tmp.bib')
        tmp = open(full_path, 'wb')
        tmp.write(source)
        tmp.close()       
        return _bibtex.open_file(full_path, 0) # 0: the 'strict' parameter

    def mapp_entries(self, entries):
        """
        turns a list of _bibtex entry objects into a list of dictionaries
        """
        return [self.mapp_entry(entry) for entry in entries]
    
    def mapp_entry(self, entry):
        """
        turn _bibtex's entry object into a simple dictionary
        to be passed to BibliographyEntry's edit method
        """
        result = {}
        authorlist = authorURLlist = []

        result['pid'] = entry[0]
        result['reference_type'] = entry[1].capitalize() + 'Reference'

        raw_items = entry[4]
        
        for key in raw_items.keys():
            result[key] = self.fixEncoding(raw_items[key][2])

        if raw_items.has_key('author'):
            result['authors'] = self.makeAuthorList(raw_items['author'][3])

        if raw_items.has_key('authorurls'):
            authorURLlist = result['authorurls'].split('and ')
            index = 0
            for url in authorURLlist:
                # restore a potentially lost tilde
                url = url.strip().replace(' ','~')
                result['authors'][index]['homepage'] = url
                index += 1
                
        # some renaming
        result['publication_year'] = result.get('year', '')
        result['publication_month'] = result.get('month', '')
        result['publication_url'] = result.get('url', '').replace(' ', '~')

        return result

    def fixEncoding(self, text):
        """primitive (not fool prove) test to ensure utf-8 encoding"""
        if type(text) not in StringTypes:
            return text
        target_enc = 'utf-8' # replace with site encoding
        if type(text) == type(u''):
            return text.encode(target_enc)
        source_enc = ['ascii', 'latin-1', 'utf-8']
        for enc in source_enc:
            try:
                return unicode(text, enc).encode(target_enc)
            except UnicodeDecodeError:
                pass
        return text # unchanged if everything fails

    def makeAuthorList(self, raw_list):
        """turns the author list into a list of dictionaries"""
        alist = []
        for a in raw_list:
            adict = {'firstname':self.fixEncoding(a[1]),
                     'lastname':self.fixEncoding(a[2]),
                     }
            alist.append(adict)
        return alist


 # Class instanciation
InitializeClass(PyBlBibtexParser)

   
def manage_addPyBlBibtexParser(self, REQUEST=None):
    """ """
    try:
        if not HAS_PYBLIOGRAPHER:
            raise ImportError
        self._setObject('pyblbibtex', PyBlBibtexParser())
    except ImportError:
        return MessageDialog(
            title='Bibliography tool warning message',
            message='The parser you attempted to add needs ' \
            'the _bibtex module from pybliographer.',
            action='manage_main')    
    except:
        return MessageDialog(
            title='Bibliography tool warning message',
            message='The parser you attempted to add already exists.',
            action='manage_main')    
    return self.manage_main(self, REQUEST)



## Script (Python) "bibliography_export"
##bind container=container
##bind context=context
##bind namespace=
##bind script=script
##bind subpath=traverse_subpath
##parameters=format='BibTex', output_encoding=None, eol_style=False
##title=
##
request = container.REQUEST
RESPONSE =  request.RESPONSE

if not format: return None

RESPONSE.setHeader('Content-Type', 'application/octet-stream')
RESPONSE.setHeader('Content-Disposition',
                   'attachment; filename=%s' %\
                   context.getId() + '.' + format)

bibtool = context.portal_bibliography
return bibtool.render(context, format=format, output_encoding=output_encoding, msdos_eol_style=eol_style)



from  bibtex import  BibtexParser, manage_addBibtexParser
from  pyblbibtex import  PyBlBibtexParser, manage_addPyBlBibtexParser
from  medline import  MedlineParser, manage_addMedlineParser
from  ibss import IBSSParser, manage_addIBSSParser
from  isbn import ISBNParser, manage_addISBNParser
from  xml import XMLParser, manage_addXMLParser
from  ris import RISParser, manage_addRISParser
from  endnote import EndNoteParser, manage_addEndNoteParser
from citationmanager import CitationManagerParser, \
     manage_addCitationManagerParser

def initialize(context):
    context.registerClass(BibtexParser,
                          constructors = (manage_addBibtexParser,),
                          ) 
    context.registerClass(PyBlBibtexParser,
                          constructors = (manage_addPyBlBibtexParser,),
                          ) 
    context.registerClass(MedlineParser,
                          constructors = (manage_addMedlineParser,),
                          ) 
    context.registerClass(IBSSParser,
                          constructors = (manage_addIBSSParser,),
                          )
    context.registerClass(EndNoteParser,
                          constructors = (manage_addEndNoteParser,),
                          )
    context.registerClass(RISParser,
                          constructors = (manage_addRISParser,),
                          )
    context.registerClass(XMLParser,
                          constructors = (manage_addXMLParser,),
                          )
    context.registerClass(ISBNParser,
                          constructors = (manage_addISBNParser,),
                          )
    context.registerClass(CitationManagerParser,
                          constructors = (manage_addCitationManagerParser,),
                          )



## Python Script "getImportReports"
##bind container=container
##bind context=context
##bind namespace=
##bind script=script
##bind subpath=traverse_subpath
##parameters=
##title=
##
reports=[]
report=[]
try:
    import_report = context.import_report
except AttributeError:
    # context must be an ATBiblioList
    import_report = context.getAssociatedBibFolder().import_report
report_lines = import_report.split('\n')
for line in report_lines:
    if line != '':
        if line != '='*30:
           report.append(line)
        else:
           reports.append(tuple(report))
           report = []
return tuple(reports)



############################################################################
#                                                                          #
#             copyright (c) 2004 ITB, Humboldt-University Berlin           #
#             written by: Raphael Ritz, r.ritz@biologie.hu-berlin.de       #
#                                                                          #
############################################################################

"""ISBNParser (via Amazon) class"""

# Python stuff
import re, string
from types import StringTypes

# Zope stuff
from Globals import InitializeClass
from App.Dialogs import MessageDialog

# CMF stuff
from Products.CMFCore.utils import getToolByName

# Bibliography stuff
from Products.CMFBibliographyAT.tool.parsers.base \
     import IBibliographyParser, BibliographyParser
from Products.CMFBibliographyAT.tool.parsers.base \
     import EntryParseError


class ISBNParser(BibliographyParser):
    """
    A specific parser to process a list of ISBN numbers. Information will be retrieved from Amazon Webservices.
    """

    __implements__ = (IBibliographyParser ,)

    meta_type = "ISBN Parser"

    format = {'name':'ISBN',
              'extension':'isbn'}

    def __init__(self,
                 id = 'isbn',
                 title = "ISBN parser via Amazon",
                 delimiter = '\n', # split on newlines
                 pattern = r'(^.{0,4}- )', # obsolete
                 flag = re.M):
        """
        initializes including the regular expression patterns
        """
        self.id = id
        self.title = title
        self.setDelimiter(delimiter)
        self.setPattern(pattern, flag)


    # Here we need to provide 'isAvailable', 'checkFormat' and 'parseEntry'

    def isAvailable(self):
    
        # in order not to get confused with ATAmazon, we quickly query the 
        # quickinstaller for the product name
        qi = getToolByName(self, 'portal_quickinstaller')
        amazontool_installed = qi.isProductInstalled('AmazonTool')
        # and, of course, we need a valid license key in the AmazonTool setup...
        amazon_tool = getToolByName(self, 'amazon_tool', False)
        return amazontool_installed and amazon_tool.hasValidLicenseKey()

    def checkFormat(self, source):
        """
        is this my format?
        """
        # are these ISBN numbers?
        # naive check: does the first line contain at least 8 digits
        ## rr: can definitively be improved

        first_line = source.split('\n')[0]
        count = 0
        for c in first_line:
            if c.isdigit(): count += 1
        if count > 7:
            return 1
        else:
            return 0

    def parseEntry(self, entry):
        """
        parses a single entry
        
        returns a dictionary to be passed to
        BibliographyEntry's edit method
        """
        result = {}
        entry = self._normalizeEntry(entry)
        if isinstance(entry, EntryParseError):
            return entry
        amazon_tool = getToolByName(self, 'amazon_tool')
        try:
            a_result = amazon_tool.searchByASIN(asin=entry)[0]
        except Exception, detail:
            return EntryParseError(detail.__str__())
        result['reference_type'] = 'BookReference'
        result['authors'] = self._makeAuthorList(a_result.Authors)
        result['title'] = a_result.ProductName
        result['publisher'] = a_result.Manufacturer
        result['isbn'] = entry
        # not sure whether the following is correct
        pdate = a_result.ReleaseDate.split(', ')
        result['publication_year'] = pdate[1]
        result['publication_month'] = pdate[0].split(' ')[-1]

        return result

    def _makeAuthorList(self, a_list):
        authors = []
        if isinstance(a_list, StringTypes):
            a_list = [a_list]
        for each in a_list:
            author = {}
            l = each.split(' ')
            author['firstname'] = l[0]
            if len(l) == 3:
                author['middlename'] = l[1]
                author['lastname'] = l[2]
            else:
                author['middlename'] = ''
                author['lastname'] = l[1]
            authors.append(author)
        return authors

    def _normalizeEntry(self, entry):
        allowed = string.ascii_letters + string.digits
        result = []
        for c in entry:
            if c in allowed:
                result.append(c)
        if result:
            return ''.join(result)
        else:
            return EntryParseError('malformed ISBN number: %s' % entry)

InitializeClass(ISBNParser)

   
def manage_addISBNParser(self, REQUEST=None):
    """ """
    try:
        getattr(self, 'amazon_tool')
    except AttributeError:
        return MessageDialog(
            title='Bibliography tool warning message',
            message='You need to install the AmazonTool before you can install this parser.',
            action='manage_main')
    try:
        self._setObject('isbn', ISBNParser())
    except:
        return MessageDialog(
            title='Bibliography tool warning message',
            message='The parser you attempted to add already exists.',
            action='manage_main')
        
    
    return self.manage_main(self, REQUEST)



############################################################################
#                                                                          #
#             copyright (c) 2004, 2006 ITB, Humboldt-University Berlin     #
#             written by: Raphael Ritz, r.ritz@biologie.hu-berlin.de       #
#                                                                          #
############################################################################

"""EndNoteParser class"""

# Python stuff
import os

# Zope stuff
from Globals import InitializeClass
from App.Dialogs import MessageDialog
from Products.CMFCore.utils import getToolByName

# Bibliography stuff
from Products.CMFBibliographyAT.tool.parsers.base \
     import IBibliographyParser, BibliographyParser

try:
    import _bibtex
    from Products.CMFBibliographyAT.tool.parsers.pyblbibtex \
         import PyBlBibtexParser as BaseParser
except ImportError:
    from Products.CMFBibliographyAT.tool.parsers.bibtex \
         import BibtexParser as BaseParser


class EndNoteParser(BaseParser):
    """
    A specific parser to process input in EndNote's text format.
    """

    __implements__ = (IBibliographyParser ,)

    meta_type = "EndNote Parser"

    format = {'name':'EndNote',
              'extension':'end'}

    def __init__(self,
                 id = 'endnote',
                 title = "EndNote's text format parser"
                 ):
        """
        initializes including the regular expression patterns
        """
        BaseParser.__init__(self, id=id, title=title)   

    # Here we need to provide 'checkFormat' and 'preprocess'

    def isAvailable(self):
        """ test if transforming from Endnote to BibTex is possible...
        """
        bib_tool = getToolByName(self, 'portal_bibliography')
        return bib_tool.isTransformable('end', 'bib')
    
    def checkFormat(self, source):
        """
        is this my format?
        """
        teststring = source[:200].lower()
        ai = teststring.find('%T')
        ei = teststring.find('%A')
        di = teststring.find('%D')
        if ai + ei + di > -2:
            return 1
        else:
            return 0

    def preprocess(self, source):
        """
        convert EndNote to BibTeX
        """
        tool = getToolByName(self, 'portal_bibliography')
        return tool.transform(source, 'end', 'bib')
    
    # all the rest we inherit from our parent BibTeX(!) parser

InitializeClass(EndNoteParser)

   
def manage_addEndNoteParser(self, REQUEST=None):
    """ """
    try:
        self._setObject('endnote', EndNoteParser())
    except:
        return MessageDialog(
            title='Bibliography tool warning message',
            message='The parser you attempted to add already exists.',
            action='manage_main')    
    return self.manage_main(self, REQUEST)



##########################################################################
#                                                                        #
#          copyright (c) 2005  LOGILAB S.A. (Paris, FRANCE)              #
#          http://www.logilab.fr/ -- mailto:contact@logilab.fr           #
#                                                                        #
##########################################################################

# Zope stuff
import os.path, copy
from Globals import InitializeClass, MessageDialog, PersistentMapping
from Acquisition import Implicit
from ZODB.PersistentList import PersistentList
from AccessControl import ClassSecurityInfo

# CMF stuff
from Products.CMFCore.permissions import View, ManagePortal
from Products.CMFCore.utils import getToolByName
from Products.PageTemplates.PageTemplateFile import PageTemplateFile
from Products.Archetypes.ArchetypeTool import ArchetypeTool
from Products.Archetypes.utils import shasattr

class DuplicatesCriteriaManager(Implicit):
    """ An object that contains :
    _criteria {bib_type : [criteria_name]}:
              contains all possible criteria (meta-data) for each bibliography type
             
    duplicates_criteria {bib_type : [criteria] } :
          contains criteria to be checked
          for duplication while importing for each bibliography type
     
    _nonmeta_criteria = ( 'reference_type', ) : contains criteria not
         comprised in meta_data

    _criteria_names = {'publication_authors' : 'authors'} : contains
         mappings for meta_data that need a special treatement like 'authors'

    _ignored_criteria = (critrias) : contains the names of  same meta_data
         like 'id' or 'allowDiscussion' to be filtered

    IMPORTANT : only 'authors','reference_type', 'publication_year',
      'title' are fully functional for now
    """
    
    _nonmeta_criteria = ( 'reference_type', )

    _default_duplicates_criteria  = (
        #FIXME - we may need a default value for these
                          'authors',
                          'reference_type',
                          'publication_year',
                          'title',)
                          
    _criteria_names = {'publication_authors' : 'authors'}

    _ignored_criteria = ('allowDiscussion','id', )

    _www = os.path.join(os.path.dirname(__file__), 'www')

    security = ClassSecurityInfo()

    manage_options = (
        { 'label'  : 'Criteria',
          'action' : 'manageImportCriteria',
          },
        )

    security.declareProtected(ManagePortal,
                              'manageImportCriteria')
    manageImportCriteria = PageTemplateFile('manage_criteria', _www)

    def __init__(self):
        self._criteria = PersistentMapping()
        self.duplicates_criteria = PersistentMapping()
        self.criteriaUpdated = False
 
    def allCriteria(self, bib_type=None):
    
	# migrate CMFBAT v0.8 duplicates engine, mending a linguistic fault-pas
	if shasattr(self, '_criterias'):
	    print 'CMFBibliographyAT: performing duplicates engine property update - v0.8 -> v0.9 (allCriteria of %s)' % '/'.join(self.getId())
	    self._criteria = PersistentMapping()
	    self.initCriteria()
	    try: delattr(self, '_criterias')
	    except: pass
	    
	# this should have been performed by __init__ but after some product migrations we might want to check it here again
        if not shasattr(self, '_criteria'):
	    self._criteria = PersistentMapping()
	if not shasattr(self, 'duplicates_criteria'):
	    self.duplicates_criteria = PersistentMapping()
	
	# first call? initialize self._criteria (available duplicates criteria per reference type)
	if not self._criteria:
            self.initCriteria()

	# always init criteria, during development, schema changes, etc.
        self.initCriteria()
        if bib_type:
            try:
        	self._criteria[bib_type]
            except KeyError:
                return False
	    	
            return self._criteria[bib_type]
	else:
    	    critKeys = self._criteria.keys()
    	    critKeys.sort()
            return [(key, self._criteria[key]) for key in critKeys]

    # fixing a linguistic fault-pas
    allcriterias = allCriteria

    def sortCriteriaByTitles(self, criteria, i18n_domain='cmfbibliographyat'):
        """ take a list of criteria tuples (as returned by allCriteria) 
            and sort it according to their i18n names. used for nice display on screen
        """
        # sorting reference types by their i18n titles
        ref_types_i18n = [ (self.translate(domain='plone', msgid=criterion[0], default=criterion[0]), criterion[0]) for criterion in criteria ]
        ref_types_i18n.sort()
        ref_types = [ t[1] for t in ref_types_i18n ]

        # turning criteria into a dictionary
        criteria_dict = {}
        for ref_type in ref_types:
            criteria_dict[ref_type] = [ t[1] for t in criteria if t[0] == ref_type ][0]

        # sorting fields for each ref_type, standard fields always preceed non-standard fields
        standard_fields = ['authors', 'publication_year', 'title', 'reference_type',]
        for ref_type in ref_types:
            nonstandard_fields_i18n = [ (self.translate(domain=i18n_domain, msgid='label_%s' % field, default=field), field) for field in criteria_dict[ref_type] if field not in standard_fields ]
            nonstandard_fields_i18n.sort()
            nonstandard_fields = [ f[1] for f in nonstandard_fields_i18n ]
            criteria_dict[ref_type] = standard_fields + nonstandard_fields
            
        return [ (t, tuple(criteria_dict[t])) for t in ref_types ]

    def initCriteria(self):
        """ initialize the dictionary of import criteria
        for each bibliography type
        """
	# this is a migration 0.8 -> 0.9 fix:
	if not shasattr(self, '_criteria'):
	    self._criteria = PersistentMapping()
	bib_tool = getToolByName(self, 'portal_bibliography')
        has = self._criteria_names.has_key
        for bib_type in bib_tool.getBibliographyContentTypes():
	    bibname = bib_type['name']
            self._criteria[bibname] = [criteria for criteria in self._nonmeta_criteria]
            #adds all meta_data as criteria for each bibliography type
            for field in bib_type['schema'].fields():
                field_name = field.getName()
                if field_name in self._ignored_criteria:
                    continue
		if not shasattr(field, 'is_duplicates_criterion'):
		    continue
		if not field.is_duplicates_criterion:
		    continue
                if has(field_name):
                    self._criteria[bibname].append(self._criteria_names[field_name])
                else :
                    self._criteria[bibname].append(field_name)
            self._criteria[bibname].sort()
            self._criteria[bibname] = tuple(self._criteria[bibname])                

    # fixing a linguistic fault-pas
    initCriterias = initCriteria
  
    def manage_changeCriteria(self, REQUEST):
        """Changes all criteria for a bibliography type given,
           called by management screen
         """
        reference_type = REQUEST.get('bibtype')
        has = REQUEST.has_key
	if reference_type != 'all':
	    reference_types = [reference_type]
	else:
	    bib_tool = getToolByName(self, 'portal_bibliography')
	    reference_types = bib_tool.getReferenceTypes()
	    
	for reference_type in reference_types:
	    criteria = []
	    for criterion in self._criteria[reference_type]:
        	# the form bibtype_criterion is not useful for now, but it was set up
        	# in case we want to use one submit input for all the bibliography types
	        if has("%s_%s" % (reference_type, criterion)):
    	            criteria.append(criterion)
	    try: self.setCriteriaForType(reference_type, criteria)
            except:
	        return MessageDialog(
            	    title  ='Warning!',
           	    message='Your changes have not been saved',           
           	    action ='manageImportCriteria')

        return MessageDialog(
           	title  ='Success!',
            	message='Your changes have been saved',
            	action ='manageImportCriteria'
        	)
		
    # fixing a linguistic fault-pas
    manage_changeCriterias = manage_changeCriteria

    def setCriteriaForType(self, bib_type, criteria):
        """update criteria for a bibliography type given"""
        self.duplicates_criteria[bib_type] = PersistentList(criteria)
        #FIXME may need to check if any change has been done
        self.criteriaUpdated = True
        return True

    # fixing a linguistic fault-pas
    setCriteriasForType = setCriteriaForType

    def setCriteria(self, duplicates_criteria):
        """update criteria for all bibliography types"""
        if not duplicates_criteria:
	    duplicates_criteria = {}
	    bib_tool = getToolByName(self, 'portal_bibliography')
	    for key in bib_tool.getReferenceTypes():
		duplicates_criteria[key] = [] 
	self.duplicates_criteria = PersistentMapping(duplicates_criteria)
        self.criteriaUpdated = True

    # fixing a linguistic fault-pas
    setCriterias = setCriteria
   
    def isCriterionSelected(self, bib_type, criterion):
	return (criterion in self.getSelectedCriteria(bib_type)) and True or False

    def isNonMetaCriterion(self, criterion):
        return criterion in self._nonmeta_criteria
	
    # fixing a linguistic fault-pas
    isCriteriaSelected = isCriterionSelected

    def getSelectedCriteria(self, bib_type = None):

	# migrate CMFBAT v0.8 duplicates engine, mending a linguistic fault-pas
	if shasattr(self, 'imp_criterias'):
	    print 'CMFBibliographyAT: performing duplicates engine property update - v0.8 -> v0.9 (getSelectedCriteria of %s)' % '/'.join(self.getId())
	    self.duplicates_criteria = PersistentMapping()
	    self.duplicates_criteria = copy.deepcopy(self.imp_criterias)
	    self.criteriaUpdated = self.criteriasUpdated
	    try: delattr(self, 'imp_criterias')
	    except: pass
	    try: delattr(self, 'criteriasUpdated')
	    except: pass

	# first call? initialize self.duplicates_criteria
	bib_tool = getToolByName(self, 'portal_bibliography')
	if not shasattr(self, 'duplicates_criteria') or not self.duplicates_criteria:

	    if self.getId() == bib_tool.getId():
		for reference_type in bib_tool.getReferenceTypes():
	    	    self.duplicates_criteria[reference_type] = PersistentList(self._default_duplicates_criteria)
		self.criteriaUpdated = True
	    else:
    		self.duplicates_criteria = bib_tool.getSelectedCriteria()
		self.criteriaUpdated = True
			
	if not shasattr(self, '_criteria') or not self._criteria:
	    self.initCriteria()	
		
	# make sure, our selected criteria are in sync with available criteria
	duplicates_criteria = {}
	for key in self._criteria.keys():
	    duplicates_criteria[key] = [ criterion for criterion in self._criteria[key] if self.duplicates_criteria.has_key(key) and (criterion in self.duplicates_criteria[key]) ]
	
        if bib_type:
            try:
        	duplicates_criteria[bib_type]
            except KeyError:
                return False
            return duplicates_criteria[bib_type]
	else:
            return duplicates_criteria

    # fixing a linguistic fault-pas
    getSelectedCriterias = getSelectedCriteria
    
InitializeClass(DuplicatesCriteriaManager)



#! /usr/bin/python
#
# this script will 
# 
#   o read (only) one file
#   o replace tabs with spaces
#   o removed CRs
#   o write clean code to stdout
#

import sys, os.path
tabsize = 8

if len(sys.argv) < 2:
    print "Usage: %s INPUT" % (os.path.basename(sys.argv[0]))
    sys.exit(1)

data = open(sys.argv[1]).read()
count = 0

for ch in data:
    if ch == '\n':
	    sys.stdout.write(ch)
	    count = 0
    elif ch == '\r':
            pass    
    elif ch == '\t':
	    newcount = count + tabsize
	    newcount -= newcount % tabsize
	    for i in range(newcount - count):
	        sys.stdout.write(' ')
	        count = newcount
    else:
	    sys.stdout.write(ch)
	    count += 1
										



import Products.CMFBibliographyAT.content.folder
import Products.CMFBibliographyAT.content.article
import Products.CMFBibliographyAT.content.book
import Products.CMFBibliographyAT.content.booklet
import Products.CMFBibliographyAT.content.conference
import Products.CMFBibliographyAT.content.inbook
import Products.CMFBibliographyAT.content.incollection
import Products.CMFBibliographyAT.content.inproceedings
import Products.CMFBibliographyAT.content.manual
import Products.CMFBibliographyAT.content.mastersthesis
import Products.CMFBibliographyAT.content.misc
import Products.CMFBibliographyAT.content.phdthesis
import Products.CMFBibliographyAT.content.preprint
import Products.CMFBibliographyAT.content.proceedings
import Products.CMFBibliographyAT.content.techreport
import Products.CMFBibliographyAT.content.unpublished
import Products.CMFBibliographyAT.content.webpublished
import Products.CMFBibliographyAT.content.pdf



##################################################
#                                                #
#    Copyright (C), 2004, Raphael Ritz           #
#    <r.ritz@biologie.hu-berlin.de>              #
#                                                #
#    Humboldt University Berlin                  #
#                                                #
##################################################

import os, sys

if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))

from Testing import ZopeTestCase
from Products.CMFPlone.tests import PloneTestCase
from Products.CMFBibliographyAT.tests import setup

class TestBibliographyTool(PloneTestCase.PloneTestCase):
    '''Test the reference types'''

    def afterSetUp(self):
        pass

    # some utility methods

    def getEmptyBibFolder(self):
        uf = self.folder
        uf.invokeFactory(type_name = "BibliographyFolder",
                         id = 'bib_folder')
        return getattr(uf, 'bib_folder')

    def getPopulatedBibFolder(self):
        bf = self.getEmptyBibFolder()
        med_source = open('medline_test.med', 'r').read()
        bf.processImport(med_source, 'medline_test.med')
        return bf   

    # the individual tests

    def testGetReferenceTypes(self):
        bibtool = self.portal.portal_bibliography
        ref_types = list(bibtool.getReferenceTypes())
        ref_types.sort()
        # the expected types
        from Products.CMFBibliographyAT.config import REFERENCE_TYPES
        expected_types = list(REFERENCE_TYPES)
        expected_types.sort()
        self.assertEqual(ref_types, expected_types)

    def testImportFormats(self):
        bibtool = self.portal.portal_bibliography
        expected_names = ['BibTeX', 'Medline']
        names = bibtool.getImportFormatNames()
        # This makes sure we only check for the required types
        # IBSS, ISBN, PyBl might be present aswell
        names = [name for name in expected_names if name in names]
        names.sort()
        self.assertEqual(names, expected_names)
        expected_extensions = ['bib', 'med']
        extensions = bibtool.getImportFormatExtensions()
        # Filter, same as for names
        extensions = [ext for ext in expected_extensions if ext in extensions]
        extensions.sort()
        self.assertEqual(extensions, expected_extensions)

    def testExportFormats(self):
        bibtool = self.portal.portal_bibliography
        names = bibtool.getExportFormatNames()
        names.sort()
        expected_names = ['BibTeX', 'EndNote', 'PDF', 'RIS', 'XML (MODS)']
        extensions = bibtool.getExportFormatExtensions()
        extensions.sort()
        expected_extensions = ['bib', 'end', 'pdf', 'ris', 'xml']
        self.assertEqual(names, expected_names)
        self.assertEqual(extensions, expected_extensions)

    # end of the individual tests

def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestBibliographyTool))
    return suite

if __name__ == '__main__':
    framework()



#! /usr/bin/python
#
# this script detects
#
#   o tabs
#   o CRs
#
# the script accepts a list of files (or file wildcards) 
# as sysargs...
#

import sys, os.path, string
tabsize = 8

if len(sys.argv) < 2:
    print "Usage: %s INPUT" % (os.path.basename(sys.argv[0]))
    sys.exit(1)

for filename in sys.argv[1:]:
    try:
        data = open(filename).read()
	count = 0

        if (string.find('\t', data) != -1) and (len(data) > 0):
    	    print 'Found tabs in file: %s.' % filename
	if (string.find('\r', data) != -1) and (len(data) > 0):
	    print 'Found CRs in file: %s' % filename

    except IOError:
	pass	    



############################################################################
#                                                                          #
#             copyright (c) 2003 ITB, Humboldt-University Berlin           #
#             written by: Raphael Ritz, r.ritz@biologie.hu-berlin.de       #
#                                                                          #
############################################################################

"""BibtexRenderer class"""

# Python stuff
import re, types, string
from urllib import unquote

# Zope stuff
from Globals import InitializeClass
from App.Dialogs import MessageDialog
from Products.BTreeFolder2.BTreeFolder2 import BTreeFolder2Base

# CMF stuff
from Products.CMFCore.utils import getToolByName

# Archetypes stuff
from Products.Archetypes.utils import shasattr

# Bibliography stuff
from Products.CMFBibliographyAT.tool.renderers.base \
     import IBibliographyRenderer, BibliographyRenderer
from Products.CMFBibliographyAT.config import FOLDER_TYPES as BIBFOLDER_TYPES

from Products.CMFBibliographyAT.utils import _encode, _decode, _default_encoding
from Products.CMFBibliographyAT.encodings import _utf8enc2latex_mapping, UNICODE_ENCODINGS


_entity_mapping = {'&mdash;':'{---}',
                   '&ndash;':'{--}',
                   }

class BibtexRenderer(BibliographyRenderer):
    """
    A specific renderer to export bibliographical references in BiBTeX format. 
    Note: If the output encoding of the BiBTeX renderer is not unicode capable, 
    all non-ASCII characters will be resolved to LaTeX entities.
    """

    __implements__ = (IBibliographyRenderer ,)

    meta_type = "Bibtex Renderer"

    format = {'name':'BibTeX',
              'extension':'bib'}

    def __init__(self,
                 id = 'bibtex',
                 title = "BibTeX renderer"):
        """
        initializes only id and title
        """
        self.id = id
        self.title = title

    def render(self, objects, output_encoding=None, title_force_uppercase=False, msdos_eol_style=False, **kwargs):
        """
        renders a bibliography object in BiBTex format
        """
        resolve_unicode = output_encoding not in UNICODE_ENCODINGS

        if type(objects) not in (types.ListType, types.TupleType):
            objects = [objects]
        bib_tool = getToolByName(objects[0], 'portal_bibliography')
        ref_types = bib_tool.getReferenceTypes()
        filter = {'portal_type' : ref_types}
        # proecssing a BibFolder's contents
        if objects[0].portal_type in BIBFOLDER_TYPES:
            # tim2p: Begin some memory optimization for BTree-ish folder types.
            # There is an element of hack to this. While it relies on a public
            # method, that method is on a private ('_tree') attribute.
            # Well, it saves me 90MB of RAM for my c.700 ref database, so I'm happy :).
            if isinstance(objects[0], BTreeFolder2Base):
                with_btree_memory_efficiency = True
                entries = objects[0]._tree.itervalues()
            else:
                with_btree_memory_efficiency = False
                entries = objects[0].contentValues(filter=filter)
            rendered = []
            for entry in entries:
                # The first condition is just a minor performance tweak. If we are not
                # using btree_memory_efficiency, then the wrong portal types will already
                # be filtered by the contentValues call above.
                if with_btree_memory_efficiency and entry.portal_type not in ref_types:
                    continue
                if with_btree_memory_efficiency:
                    # _tree.itervalues() returns unwrapped objects, but renderEntry needs
                    # context-aware objects, so we wrap it.
                    # Check to see if the object has been changed (elsewhere in the
                    # current transaction/request.
                    changed = getattr(entry, '_p_changed', False)
                    deactivate = not changed
                    entry = entry.__of__(objects[0])
                bibtex_string = self.renderEntry(entry,
                                                 resolve_unicode=resolve_unicode,
                                                 title_force_uppercase=title_force_uppercase,
                                                 msdos_eol_style=msdos_eol_style,
                                                )
                rendered.append(bibtex_string)
                if with_btree_memory_efficiency and deactivate:
                    # We now 'unload' the entry from the ZODB object cache to reclaim its memory.
                    # See http://wiki.zope.org/ZODB/UnloadingObjects for details.
                    # XXX In fact, there are probably no bad side-effects to making this call
                    # even if not with_btree_memory_effiency. However, I want my patch to be
                    # as non-intrusive as possible, so I don't do that now.
                    entry._p_deactivate()
            return self._convertToOutputEncoding(''.join(rendered),
                                                 output_encoding=output_encoding)
        
        # processing a single or a list of BibRef Items
        else:
            rendered = []
            for object in objects:
                if object.portal_type in ref_types:
                    rendered.append(self.renderEntry(object, resolve_unicode=resolve_unicode, title_force_uppercase=title_force_uppercase, msdos_eol_style=msdos_eol_style))
            return self._convertToOutputEncoding(''.join(rendered),
                                                 output_encoding=output_encoding)        
            
        return ''

    def renderEntry(self, entry, resolve_unicode=False, title_force_uppercase=False, msdos_eol_style=False):
        """
        renders a BibliographyEntry object in BiBTex format
        """
        bib_key = self._validKey(entry)
        bibtex = "\n@%s{%s," % \
                 (entry.meta_type[:-9], bib_key)
        # [:-9] assumes we have a "...Reference" type 
        if shasattr(entry, 'editor_flag') and \
               entry.getFieldValue('editor_flag'):
            bibtex += "\n  editor = {%s}," % \
                      entry.Authors(sep=' and ',
                                    lastsep=' and ',
                                    format="%L, %F %M",
                                    abbrev=0,
                                    lastnamefirst=0)
        else:                            
            bibtex += "\n  author = {%s}," % \
                      entry.Authors(sep=' and ',
                                    lastsep=' and ',
                                    format="%L, %F %M",
                                    abbrev=0,
                                    lastnamefirst=0)
        aURLs = self.AuthorURLs(entry)
        if aURLs.find('http') > -1:
            bibtex += "\n  authorURLs = {%s}," % aURLs
        if title_force_uppercase:
            bibtex += "\n  title = {%s}," % self._braceUppercase(entry.Title())
        else:
            bibtex += "\n  title = {%s}," % entry.Title()
        bibtex += "\n  year = {%s}," % entry.getPublication_year()
        url = entry.aq_base.getURL()
        if url: bibtex += "\n  URL = {%s}," % url
        bibtex += "\n  abstract = {%s}," % entry.getAbstract()

        if hasattr(entry, 'source_fields') and entry.source_fields:
            source_fields = list(entry.source_fields)
            field_values = [entry.getFieldValue(name) \
                            for name in source_fields]
            if 'publication_type' in source_fields:
                source_fields[source_fields.index('publication_type')] = 'type'
            for key, value in zip(source_fields, field_values):
                if value:
                    bibtex += "\n  %s = {%s}," % (key, value)

        kws = ', '.join(entry.Subject())
        if kws:
            bibtex += "\n  keywords = {%s}," % kws
        note = entry.getNote()
        if note:
            bibtex += "\n  note = {%s}," % note
        annote = entry.getAnnote()
        if annote:
            bibtex += "\n  annote = {%s}," % annote
        if bibtex[-1] == ',':
            bibtex = bibtex[:-1] # remove the trailing comma
        bibtex += "\n}\n"
        bibtex = self._normalize(bibtex, resolve_unicode=resolve_unicode)

        # leave these lines to debug _utf8enc2latex_mapping problems (for now)
        try:
            if resolve_unicode: debug = _decode(bibtex).encode('latin-1')
        except UnicodeEncodeError:
            print 'UnicodeEncodeError (latin-1): caused by object with ID: %s' % bib_key

        if msdos_eol_style:
            bibtex = bibtex.replace('\n', '\r\n')
        return bibtex

    def _validKey(self, entry):
        """
        uses the Zope object id but
        removes characters not allowed in BibTeX keys
        """
        raw_id = entry.getId()
        # This substitution is based on the description of cite key restrictions at
        # http://bibdesk.sourceforge.net/manual/BibDesk%20Help_2.html
        return re.sub('[ "@\',\\#}{~%&$^]', '', raw_id)

    def AuthorURLs(self, entry):
        """a string with all the known author's URLs;
        helper method for bibtex output"""
        a_list = entry.getAuthorList()
        a_URLs = ''
        for a in a_list:
            url = a.get('homepage', ' ')
            a_URLs += "%s and " % url
        return a_URLs[:-5]

    def _normalize(self, text, resolve_unicode=True):
        text.replace('\\', '\\\\')
        text = self._resolveEntities(text)
        if resolve_unicode:
            text = self._resolveUnicode(text)
        text = self._escapeSpecialCharacters(text)
        return text

    def _resolveEntities(self, text):
        for entity in _entity_mapping.keys():
            text = text.replace(entity, _entity_mapping[entity])
        return text

    def _resolveUnicode(self, text):
        for unichar in _utf8enc2latex_mapping.keys():
            text = _encode(_decode(text).replace(unichar, _utf8enc2latex_mapping[unichar]))
        text = _encode(_decode(text).replace(r'$}{$',''))
        return text

    def _braceUppercase(self, text):
        for uc in string.uppercase:
            text = text.replace(uc, r'{%s}' % uc)
        return text

    def _escapeSpecialCharacters(self, text):
        """
        latex escaping some (not all) special characters
        """
        text.replace('\\', '\\\\')
        escape = ['~', '#', '&', '%', '_']
        for c in escape:
            text = text.replace(c, '\\' + c )
        return text

InitializeClass(BibtexRenderer)

   
def manage_addBibtexRenderer(self, REQUEST=None):
    """ """
    try:
        self._setObject('bibtex', BibtexRenderer())
    except:
        return MessageDialog(
            title='Bibliography tool warning message',
            message='The renderer you attempted to add already exists.',
            action='manage_main')
    return self.manage_main(self, REQUEST)



##########################################################################
#                                                                        #
#           copyright (c) 2005 ITB, Humboldt-University Berlin           #
#           written by: Raphael Ritz, r.ritz@biologie.hu-berlin.de       #
#                                                                        #
##########################################################################

from Globals import InitializeClass
from AccessControl import ClassSecurityInfo
from Products.CMFCore.utils import getToolByName
from Products.Archetypes.Marshall import Marshaller

class BibtexMarshaller(Marshaller):

    security = ClassSecurityInfo()
    security.declareObjectPrivate()
    security.setDefaultAccess('deny')

    def demarshall(self, instance, data, **kwargs):
        if kwargs.has_key('file'):
            if not data:
                data = kwargs['file'].read()
            del kwargs['file']
        bibtool = getToolByName(instance, 'portal_bibliography')
        entry = bibtool.getEntries(data, 'bib')[0]
        instance.setAuthors(entry.get('authors',[]))
        instance.edit(**entry)

    def marshall(self, instance, **kwargs):
        bibtool = getToolByName(instance, 'portal_bibliography')
        data = bibtool.render(instance, 'bib')
        length = len(data)
        content_type = 'application/x-bibtex'
        return (content_type, length, data)

InitializeClass(BibtexMarshaller)




##################################################
#                                                #
#    Copyright (C), 2004, Raphael Ritz           #
#    <r.ritz@biologie.hu-berlin.de>              #
#                                                #
#    Humboldt University Berlin                  #
#                                                #
##################################################

import os, sys

if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))

from Testing import ZopeTestCase
from Products.CMFPlone.tests import PloneTestCase
from Products.CMFBibliographyAT.tests import setup

class TestCMFBibliographyATInstall(PloneTestCase.PloneTestCase):
    '''Test the CMFBibliographyAT installation'''

    def afterSetUp(self):
        pass

    # the individual tests
    
    def testSkinInstallation(self):
        st = self.portal.portal_skins
        self.failUnless('bibliography' in st.objectIds())

    def testToolInstalltion(self):
        root = self.portal
        self.failUnless('portal_bibliography' in root.objectIds())

    def testParserFolderInstallation(self):
        bibtool = self.portal.portal_bibliography
        self.failUnless('Parsers' in bibtool.objectIds())

    def testRendererFolderInstallation(self):
        bibtool = self.portal.portal_bibliography
        self.failUnless('Renderers' in bibtool.objectIds())

    def testBibtexParserInstallation(self):
        parsers = self.portal.portal_bibliography.Parsers
        self.failUnless('bibtex' in parsers.objectIds())

    def testMedlineParserInstallation(self):
        parsers = self.portal.portal_bibliography.Parsers
        self.failUnless('medline' in parsers.objectIds())

    def testBibtexRendererInstallation(self):
        renderers = self.portal.portal_bibliography.Renderers
        self.failUnless('bibtex' in renderers.objectIds())

    def testBibFolderInstallation(self):
        ttool = self.portal.portal_types
        self.failUnless('BibliographyFolder' in ttool.objectIds())

    def testReferenceTypesInstallation(self):
        ttool = self.portal.portal_types
        # first check all listed in config
        from Products.CMFBibliographyAT.config import REFERENCE_TYPES
        for reftype in REFERENCE_TYPES:
            self.failUnless(reftype in ttool.objectIds(),
                            '%s not installed' % reftype)
        # to double check do also explicit tests
        reftypes = ['ArticleReference',
                    'BookReference',
                    'BookletReference',
                    'InbookReference',
                    'IncollectionReference',
                    'InproceedingsReference',
                    'ManualReference',
                    'MastersthesisReference',
                    'MiscReference',
                    'PhdthesisReference',
                    'PreprintReference',
                    'ProceedingsReference',
                    'TechreportReference',
                    'UnpublishedReference',
                    'WebpublishedReference',
                    ]
        for reftype in reftypes:
            self.failUnless(reftype in ttool.objectIds(),
                            '%s not installed' % reftype)


    def testCatalogExtensions(self):
        ctool = self.portal.portal_catalog

        newFieldIndexes = ['Authors', 'publication_year']
        newSchemaEntries = ['Authors', 'publication_year', 'Source']
        
        for idx in newFieldIndexes:
            self.failUnless(idx in ctool.indexes())

        for entry in newSchemaEntries:
            self.failUnless(entry in ctool.schema())

## rr: obsolte in CMF-2.0; custom tools must not be action providers
##     def testActionProvider(self):
##         atool = self.portal.portal_actions
##         self.failUnless('portal_bibliography' \
##                         in atool.listActionProviders())

    def testBibliographyViewAction(self):
        acttool = self.portal.portal_actions
        action_ids = [a.id for a in acttool.listActions()]
        self.failUnless('bibliography_search' in action_ids)

    # end of the individual tests

def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestCMFBibliographyATInstall))
    return suite

if __name__ == '__main__':
    framework()



##########################################################################
#                                                                        #
#           copyright (c) 2003 ITB, Humboldt-University Berlin           #
#           written by: Raphael Ritz, r.ritz@biologie.hu-berlin.de       #
#                                                                        #
##########################################################################

"""Techreport reference main class"""

from AccessControl import ClassSecurityInfo
from Products.CMFCore.permissions import View

from Products.CMFBibliographyAT.config import CMFBAT_USES_LINGUAPLONE
if CMFBAT_USES_LINGUAPLONE:
    from Products.LinguaPlone.public import Schema
    from Products.LinguaPlone.public import StringField, StringWidget
else:
    from Products.Archetypes.public import Schema
    from Products.Archetypes.public import StringField, StringWidget
from Products.Archetypes.public import registerType

from Products.CMFBibliographyAT.config import PROJECTNAME
from Products.CMFBibliographyAT.content.base import BaseEntry
from Products.CMFBibliographyAT.content.schemata \
    import HeaderSchema, AuthorSchema, CoreSchema, TrailingSchema
from Products.CMFBibliographyAT.content.fields \
    import institutionField, typeField, numberField, addressField


SourceSchema = Schema((
    institutionField,
    typeField,
    numberField,
    addressField,
    ))

TechreportSchema = HeaderSchema.copy() + AuthorSchema.copy() + CoreSchema.copy() +  \
                   SourceSchema.copy() + TrailingSchema.copy()
TechreportSchema.get('authors').required = 1
TechreportSchema.get('publication_year').required = 1
TechreportSchema.get('institution').required = 1

# the default AT 'description' field shall be invisible, it is kept in sync with the 'abstract' field
TechreportSchema.get('description').widget.visible = {'view': 'invisible', 'edit': 'invisible', }


class TechreportReference(BaseEntry):
    """ content type to make reference to a technical report.
    """
    security = ClassSecurityInfo()
    archetype_name = "Techreport Reference"
    source_fields = ('institution', 'publication_type', 'number', 'address',)

    schema = TechreportSchema
    
    security.declareProtected(View, 'Source')
    def Source(self):
        """ the default technical report source format
        """
        try:

            return self.TechreportSource()

        except AttributeError:

            institution 		= self.getInstitution()
            publication_type        	= self.getPublication_type()
            number      		= self.getNumber()
            address     		= self.getAddress()
            
	    source = institution
	    if publication_type: 
		source += ', %s' % publication_type
		if number: source += '(%s)' % number
	    if address: source += ', %s' % address	
	
            return source + '.'


registerType(TechreportReference, PROJECTNAME)



from Products.Archetypes.public import listTypes
from Products.Archetypes.Extensions.utils import installTypes, install_subskin
from Products.bibTypes.config import *

from StringIO import StringIO

def install(self):
    out = StringIO()
    
    installTypes(self, out,
                 listTypes(PROJECTNAME),
                 PROJECTNAME)

    install_subskin(self, out, GLOBALS)

    print >> out, "Successfully installed %s." % PROJECTNAME
    return out.getvalue()



"""EtAl Bibliography IdCooker class"""

# Zope stuff
from Globals import InitializeClass
from App.Dialogs import MessageDialog

# CMF stuff
from Products.CMFCore.utils import getToolByName

# Bibliography stuff
from Products.CMFBibliographyAT.tool.idcookers.base \
     import IBibliographyIdCooker, BibliographyIdCooker


class EtalIdCooker(BibliographyIdCooker):
    """
    If a bibliographical reference item has one or two authors, it uses the authorname(s) + publication year to cook a reference's ID. 
    If an item has more than two authors, it uses the first author's name plus 'EtAl' plus the publication year.
    """

    __implements__ = (IBibliographyIdCooker ,)

    meta_type = "EtAl Bibliography ID Cooker"

    def __init__(self,
                 id = 'etal',
                 title = "EtAl Bibliography ID Cooker"):
        """
        initializes id and title
        """
        self.id = id
        self.title = title

    def _cookIdCore(self, ref, new_id='', **kwargs):
        """
        cooks a bibref id for one reference entry dict
        """

	# AUTHORS
	namepart = 'nobody'
	if self._refHasAuthorNames(ref):
            lastnames = []
            for each in ref['authors']:
		if each.get('lastname', None):
    	    	    lastnames.append(each['lastname'])

            if len(lastnames) > 2:
                namepart = '%sEtAl' % lastnames[0]
            elif len(lastnames) <= 2:
	        namepart = ''.join(lastnames)
    	    else:
		pass

	# PUBLICATION YEAR
	if ref.get('publication_year', None):
	    yearpart = str(ref['publication_year'])
	else:
	    yearpart = "1000"
	
	return  namepart + yearpart

InitializeClass(EtalIdCooker)


def manage_addEtalIdCooker(self, REQUEST=None):
    """ """
    try:
        self._setObject('etal', EtalIdCooker())
    except:
        return MessageDialog(
            title='Bibliography tool warning message',
            message='The id cooker you attempted to add already exists.',
            action='manage_main')
    return self.manage_main(self, REQUEST)



##########################################################################
#                                                                        #
#           copyright (c) 2003-2006 ITB, Humboldt-University Berlin      #
#           written by: Raphael Ritz, r.ritz@biologie.hu-berlin.de       #
#                                                                        #
##########################################################################

"""plone installer script"""

from StringIO import StringIO

from Products.CMFCore.utils import getToolByName

from Products.Archetypes.public import listTypes
from Products.Archetypes.Extensions.utils import installTypes, install_subskin

from Products.MimetypesRegistry import MimeTypeItem

from Products.CMFBibliographyAT.config import *
from Products.CMFBibliographyAT.tool.parsers import *
from Products.CMFBibliographyAT.tool.renderers import *
from Products.CMFBibliographyAT.tool.idcookers import *
from Products.CMFBibliographyAT.migrations import *

# helper methods to be called on tool installation

def addParsers(self):
    """
    helper method to set up a few parsers
    """
    if getattr(self.aq_explicit, 'bibtex', None) is None:
        bibparser = BibtexParser('bibtex')
        self._setObject('bibtex', bibparser)
    if getattr(self.aq_explicit, 'medline', None) is None:
        medparser = MedlineParser('medline')
        self._setObject('medline', medparser)
    if getattr(self.aq_explicit, 'ibss', None) is None:
        ibssparser = IBSSParser('ibss')
        self._setObject('ibss', ibssparser)
    if getattr(self.aq_explicit, 'isbn', None) is None:
        isbnparser = ISBNParser('isbn')
        self._setObject('isbn', isbnparser)
    if getattr(self.aq_explicit, 'endnote', None) is None:
        endparser = EndNoteParser('endnote')
        self._setObject('endnote', endparser)
    if getattr(self.aq_explicit, 'ris', None) is None:
        risparser = RISParser('ris')
        self._setObject('ris', risparser)
    if getattr(self.aq_explicit, 'xml_mods', None) is None:
        xmlparser = XMLParser('xml_mods')
        self._setObject('xml_mods', xmlparser)
    if getattr(self.aq_explicit, 'citationmanager', None) is None:
        citationmanagerparser = CitationManagerParser('citationmanager')
        self._setObject('citationmanager', citationmanagerparser)

def addRenderers(self):
    """
    helper method to set up a few renderers
    """
    id = 'bibtex'
    renderer = BibtexRenderer(id)
    self._setObject(id, renderer)
    renderer.initializeDefaultOutputEncoding(portal_instance=self)
    id = 'endnote'
    renderer = EndRenderer(id)
    self._setObject(id, renderer)
    renderer.initializeDefaultOutputEncoding(portal_instance=self)
    id = 'ris'
    renderer = RISRenderer(id)
    self._setObject(id, renderer)
    renderer.initializeDefaultOutputEncoding(portal_instance=self)
    id = 'pdf'
    renderer = PDFRenderer(id)
    self._setObject(id, renderer)
    renderer.initializeDefaultOutputEncoding(portal_instance=self)
    id = 'xml_mods'
    renderer = XMLRenderer(id)
    self._setObject(id, renderer)
    renderer.initializeDefaultOutputEncoding(portal_instance=self)

def addIdCookers(self):
    """
    helper method to set up a id cookers
    """
    id = 'plone'
    idcooker = PloneIdCooker(id)
    self._setObject(id, idcooker)
    id = 'etal'
    idcooker = EtalIdCooker(id)
    self._setObject(id, idcooker)
    id = 'abbrev'
    idcooker = AbbrevIdCooker(id)
    self._setObject(id, idcooker)
    id = 'uid'
    idcooker = UidIdCooker(id)
    self._setObject(id, idcooker)


# tool installation
def setupTool(self, out):
    """
    adds the bibliography tool to the portal root folder
    """
    if hasattr(self, 'portal_bibliography'):
        self.manage_delObjects(['portal_bibliography'])
        out.write('Deleting old tool; make sure you repeat customizations.')
    addTool = self.manage_addProduct['CMFBibliographyAT'].manage_addTool
    addTool('Bibliography Tool', None)
    out.write("\nAdded the bibliography tool to the portal root folder.\n")
    bibtool = getToolByName(self, 'portal_bibliography')

    prep_server = PREPRINT_SERVERS
    try:
        bibtool.manage_changeProperties({
            'preprint_servers':prep_server,
            },
                                        )
        out.write("Set default properties at the bibliography tool.\n")
    except AttributeError:
        pass

    # add default parsers and renderers
    addParsers(bibtool.Parsers)
    addRenderers(bibtool.Renderers)
    addIdCookers(bibtool.IdCookers)
    out.write(
        "Added default parsers, renderers and id cookers to the bibliography tool.\n")


def addToCatalog(self, out):
    ct = getToolByName(self, 'portal_catalog')

    newFieldIndexes = ['Authors', 'publication_year']
    newSchemaEntries = ['Authors', 'publication_year',
                        'Source', 'citationLabel']
    newKeywordIndexes = ['AuthorItems']

    for idx in newFieldIndexes:
        if idx in ct.indexes():
            out.write("Found the '%s' index in the catalog.\n" % idx)
        else:
            ct.addIndex(idx, 'FieldIndex')
            out.write("Added a field index '%s' to the catalog.\n" % idx)

    for idx in newKeywordIndexes:
        if idx in ct.indexes():
            out.write("Found the '%s' index in the catalog.\n" % idx)
        else:
            ct.addIndex(idx, 'KeywordIndex')
            out.write("Added a keyword index '%s' to the catalog.\n" % idx)

    # DateIndex for the publication_date
    if 'publication_date' not in ct.indexes():
        ct.addIndex('publication_date', 'DateIndex')

    for entry in newSchemaEntries:
        if entry in ct.schema():
            out.write("Found '%s' in the catalog meta data.\n" % entry)
        else:
            ct.addColumn(entry)
            out.write("Added '%s' to the catalog meta data.\n" % entry)

def fixContentTab(self, out):
    pp = getToolByName(self, 'portal_properties', None)
    if pp and hasattr(pp, 'site_properties'):
        use_folder_tabs = pp.site_properties.getProperty('use_folder_tabs', [])
        for ftype in FOLDER_TYPES:
            if ftype not in use_folder_tabs:
                use_folder_tabs += (ftype, )
                pp.site_properties.manage_changeProperties(
                    {'use_folder_tabs' : use_folder_tabs},
                    )

def setFolderWorkflow(self, out):
    wf_tool = getToolByName(self, 'portal_workflow')
    wf_tool.setChainForPortalTypes( FOLDER_TYPES, 'folder_workflow')
    out.write("Made %s subject to 'folder_workflow'.\n" %
              ' and '.join(FOLDER_TYPES))

def addPrefsPanel(self, out):
    cp=getToolByName(self, 'portal_controlpanel', None)
    if not cp:
        out.write("No control panel found. Skipping installation of the setup panel.\n")
    else:
 	cp.addAction(id='BibliographySetup',
                     name='Bibliography Setup',
                     action='string:${portal_url}/prefs_bibliography_docs',
                     permission='Manage portal',
                     category='Products',
                     appId='CMFBibliographyAT',
                     imageUrl='bibliography_tool.png',
                     description='Configure global settings of the bibliography tool.')
        out.write("Installed the bibliography tool configuration panel.\n")
	

def addActions(self, out):
    ap=getToolByName(self, 'portal_actions')

    # Check if the old 'download' action still exists and remove it
    if ap is not None:
        new_actions = [a for a in ap._cloneActions()
                       if a.getId() != 'downloadBib']
        ap._actions = new_actions
    # Add exportBib action to the actions tool
    ## rr: this is still the old way, i.e., name instead of title;
    ## no support for description; permission instead of permissions;
    ## we ought to move to an extension
    ## profile anyway soon so I don't care; it works for now
    ap.addAction(
        id='exportBib',
        name='Export Bibliography',
        action='string:${object_url}/bibliography_exportForm',
        permission='View',
        category='document_actions',
        condition='python:portal.isBibliographyExportable(object)',
        visible=1,
        )

    ap.addAction(
        id='bibliography_search',
        name='Bibliography Search',
        # description='Site wide; sorted by year and authors',
        action='string:${portal_url}/bibliography_search_form',
        permission='View',
        category='portal_tabs',
        condition=None,
        visible=1,
        )

    ai = getToolByName(self, 'portal_actionicons', None)
    if ai is not None:

        # Add the 'export', 'import', and 'pdfdownload' action icons
	ai.addActionIcon('plone','exportBib','bibliography_export.png','Export',0)
	ai.addActionIcon('plone','import','bibliography_import.png','Import',0)
        ai.addActionIcon('plone','download_pdf','bibliography_pdffile.png','Printable file',0)

def changeDefaultViewFor(self, out, type):
    """
    changes the default view for <type> from
    'base_view' to 'bibliographyfolder_view'
    """
    fti = getattr(self.portal_types.aq_base, type, None)
    if not fti: return None
    for a in fti._actions:
        if a.id == 'view':
            a.setActionExpression('bibliographyfolder_view')
            print >> out, \
                  "Changed the %s's default view to" \
                  "'bibliographyfolder_view'." \
                  % type

def fixBibFolderView(self, out):
    """
    checks for the existance of the 'folderlisting' macro in 'base'
    changes the BibFolders's default view if not found
    """
    base_template = getattr(self, 'base', None)
    if base_template is None:
        return None
    macros = getattr(base_template, 'macros', None)
    if macros and 'folderlisting' not in macros.keys():
        changeDefaultViewFor(self, out, 'BibliographyFolder')
        changeDefaultViewFor(self, out, 'LargeBibliographyFolder')

def addCTREntry(self, out):
    """
    add a predicate for 'Bibliography' to the content type registry
    """
    ## make this configurable ???
    predicate_id = 'Bibliography_ext'
    extensions = 'bib, med, end, ris'  # comma separated list
    type_name = 'BibliographyFolder'
    
    ctr = self.content_type_registry
    predicate_ids = [p[0] for p in ctr.listPredicates()]
    if predicate_id not in predicate_ids:
        ctr.addPredicate(predicate_id, 'extension')
        out.write("Adding a '%s' predicate to the content type registry\n"\
                  % predicate_id)
    predicate = ctr.getPredicate(predicate_id)
    predicate.edit(extensions)
    ctr.updatePredicate(predicate_id, predicate, type_name)
    ctr.reorderPredicate(predicate_id, 0)

def install_transform(self, out):
    try:
        print >>out, "Adding new mimetype"
        mimetypes_tool = getToolByName(self, 'mimetypes_registry')
        newtype = MimeTypeItem.MimeTypeItem('HTML with inline citations',
            ('text/x-html-bibaware',), ('html-bib',), 0)
        mimetypes_tool.register(newtype)

        print >>out,"Add transform"
        transform_tool = getToolByName(self, 'portal_transforms')
        try:
            transform_tool.manage_delObjects(['html-to-bibaware'])
        except: # XXX: get rid of bare except
            pass
        transform_tool.manage_addTransform('html-to-bibaware',
                                           'Products.CMFBibliographyAT.transform.html2bibaware')
        if 'bibaware-to-html' not in transform_tool.objectIds():
            transform_tool.manage_addTransform('bibaware-to-html',
                                               'Products.CMFBibliographyAT.transform.bibaware2html')
        addPolicy(transform_tool, out)
    except (NameError,AttributeError):
        print >>out, "No MimetypesRegistry, bibaware text not supported."

def addPolicy(tool, out):
    target = 'text/x-html-safe'
    transform = 'html-to-bibaware'
    policies = tool.listPolicies()
    policy_keys = [p[0] for p in policies]
    if target not in policy_keys:
        print >>out, "Adding transformation policy for bibliographic " \
              "awareness of the 'text/x-html-safe' type."
        tool.manage_addPolicy(target,
                              (transform,),
                              )
    else:
        transforms = [p[1] for p in policies if p[0] == target][0]
        if transform not in transforms:
            transforms += (transform,)
            ## no API for changing a policy :-(
            tool.manage_delPolicies((target,))
            tool.manage_addPolicy(target, transforms)
            print >>out, "Adding the bibliographic transformation to " \
                  "the 'text/x-html-safe' policy."
        else:
            print >>out, "Checked transformation policy"

def autoMigrate(self, out):
    migrations = ( 
	cmfbib07to08.Migration(self, out), 
	cmfbib08to09.Migration(self, out),
    )	
    for migration in migrations:
	migration.migrate()

def install(self):
    out = StringIO()

    # the ActionIcon tool is pretty picky with already existing icons, and easily causes product install
    # to fail with a KeyError: 'Duplicate definition!'. 
    #
    # To avoid this, we try to get rid of all our icons before re-installing them... 
    # 
    removeActionIcons(self)

    setupTool(self, out)
    addPrefsPanel(self, out)
    addActions(self,out)
    addToCatalog(self,out)
    installTypes(self, out,
                 listTypes(PROJECTNAME),
                 PROJECTNAME)

    install_subskin(self, out, GLOBALS)
    install_transform(self, out)
    fixContentTab(self, out)   # for backwards compatibility
    fixBibFolderView(self, out) # for backwards compatibility
    setFolderWorkflow(self, out)
    addCTREntry(self, out)

    autoMigrate(self, out)

    print >> out, "Successfully installed %s's content types." % PROJECTNAME

    return out.getvalue()

# the uninstall need's the following functions

def removePrefsPanel(self):
    cp=getToolByName(self, 'portal_controlpanel', None)
    if cp:
        cp.unregisterApplication('CMFBibliographyAT')

def removeFromActionProviders(self):
    """
    removes portal_bibliography from the action providers
    registered with the action tool
    """
    acttool = getToolByName(self, 'portal_actions')
    if 'portal_bibliography' in acttool.listActionProviders():
        acttool.deleteActionProvider('portal_bibliography')

def removeAction(self):
    """
    removes the export action from the actions tool
    (why doesn't the quickinstaller do that?)
    """
    acttool = getToolByName(self, 'portal_actions')
    actions = list(acttool._actions)
    keep = []
    for a in actions:
        if a.id != 'exportBib':
            keep.append(a)
    acttool._actions = tuple(keep)

def removeActionIcons(self):

    ai = getToolByName(self, 'portal_actionicons')

    # the ActionIcon tool is pretty picky with already existing icons, and easily causes product install
    # to fail with a KeyError: 'Duplicate definition!'. 
    #
    # To avoid this, this function is called on install AND uninstall
    #
    #   o ('plone', 'downloadBib') -> deprecated, has been renamed to exportBib
    #   o ('plone', 'exportBib'), 
    #   o ('plone', 'download_pdf') -> just in case uninstall ai remove failed on product uninstall
    #   o ('controlpanel', 'BibliographySetup') -> in case of an error raised after adding the bibliography 
    #     tool during last product install	
    for category, id in [('plone', 'downloadBib'), ('plone', 'exportBib'), ('plone', 'import'), ('plone', 'download_pdf'), ('controlpanel', 'BibliographySetup'),]:
	try:
    	    ai.removeActionIcon(category,id)
	except KeyError:
    	    pass

def uninstall_transform(self, out):
    transform_tool = getToolByName(self, 'portal_transforms')
    try:
        transform_tool.manage_delObjects(['html-to-bibaware'])
    except:
        pass
    else:
        print >>out, "Transform removed"

def uninstall(self):
    out = StringIO()
    removeFromActionProviders(self)
    removePrefsPanel(self)
    removeActionIcons(self)
    removeAction(self)
    uninstall_transform(self, out)
    # all the rest of the cleaning we leave to quickinstaller
    print >> out, "Uninstalled CMFBibliographyAT."
    return out.getvalue()
